@@ -2,7 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2010-2011 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2010-2012 by Andrea Diamantini <adjam7 at gmail dot com>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -52,7 +52,7 @@ class REKONQ_TESTS_EXPORT NetworkAnalyzerPanel : public QDockWidget
 public:
     NetworkAnalyzerPanel(const QString &title, QWidget *parent);
 
-public slots:
+public Q_SLOTS:
     void toggle(bool enable);
     void changeCurrentPage();
 
