@@ -2,7 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2012 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2009 by Paweł Prażak <pawelprazak at gmail dot com>
 * Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
 *
@@ -33,9 +33,12 @@
 // Auto Includes
 #include "rekonq.h"
 
+// Ui Includes
+#include "ui_webappcreation.h"
+
 // Local Includes
 #include "adblockmanager.h"
-#include "bookmarkprovider.h"
+#include "bookmarkmanager.h"
 #include "downloadmanager.h"
 #include "filterurljob.h"
 #include "historymanager.h"
@@ -44,9 +47,11 @@
 #include "mainwindow.h"
 #include "opensearchmanager.h"
 #include "sessionmanager.h"
+#include "syncmanager.h"
 #include "stackedurlbar.h"
 #include "tabbar.h"
 #include "urlbar.h"
+#include "useragentmanager.h"
 #include "webtab.h"
 
 // KDE Includes
@@ -61,6 +66,7 @@
 #include <KWindowInfo>
 #include <KGlobal>
 #include <KCharsets>
+#include <KPushButton>
 
 // Qt Includes
 #include <QVBoxLayout>
@@ -103,11 +109,11 @@ Application::~Application()
         m_historyManager.clear();
     }
 
-    if (!m_bookmarkProvider.isNull())
+    if (!m_bookmarkManager.isNull())
     {
-        kDebug() << "deleting bookmark Provider";
-        delete m_bookmarkProvider.data();
-        m_bookmarkProvider.clear();
+        kDebug() << "deleting bookmark Manager";
+        delete m_bookmarkManager.data();
+        m_bookmarkManager.clear();
     }
 
     if (!m_sessionManager.isNull())
@@ -138,6 +144,20 @@ Application::~Application()
         m_adblockManager.clear();
     }
 
+    if (!m_userAgentManager.isNull())
+    {
+        kDebug() << "deleting user agent manager";
+        delete m_userAgentManager.data();
+        m_userAgentManager.clear();
+    }
+
+    if (!m_syncManager.isNull())
+    {
+        kDebug() << "deleting sync manager";
+        delete m_syncManager.data();
+        m_syncManager.clear();
+    }
+
     // TODO:
     // add a check to NOT close rekonq
     // until last download is finished
@@ -163,9 +183,9 @@ int Application::newInstance()
     // 2) Are there arguments?
     // 3) Is rekonq recovering from crash?
     // so, we have 8 possible cases...
-    bool isFirstLoad = m_mainWindows.isEmpty();
+    static bool isFirstLoad = true;
     bool areThereArguments = (args->count() > 0);
-    bool isRekonqCrashed = ReKonfig::recoverOnCrash();
+    bool isRekonqCrashed = (ReKonfig::recoverOnCrash() > 0);
     // note that isRekonqCrashed is always true if it is not the first load
     // !isFirstLoad -> isRekonqCrashed
 
@@ -175,24 +195,47 @@ int Application::newInstance()
 
     int exitValue = 1 * isFirstLoad + 2 * areThereArguments + 4 * isRekonqCrashed;
 
+    if (isFirstLoad && isSessionRestored())
+    {
+        isFirstLoad = false;
+
+        return exitValue;
+    }
+
     if (isRekonqCrashed && isFirstLoad)
     {
         loadUrl(KUrl("about:closedTabs"), Rekonq::NewWindow);
+        QTimer::singleShot(1000, mainWindow()->currentTab(), SLOT(showMessageBar()));
     }
 
     if (areThereArguments)
     {
+        // prepare URLS to load
         KUrl::List urlList;
         for (int i = 0; i < args->count(); ++i)
         {
             const KUrl u = args->url(i);
+
             if (u.isLocalFile() && QFile::exists(u.toLocalFile())) // "rekonq somefile.html" case
                 urlList += u;
             else
-                urlList += KUrl(args->arg(i));   // "rekonq kde.org" || "rekonq kde:kdialog" case
+            {
+                // "rekonq kde.org" || "rekonq kde:kdialog" case
+                UrlResolver res(args->arg(i));
+                UrlSearchList list = res.orderedSearchItems();
+                if (list.isEmpty())
+                {
+                    urlList += u;
+                }
+                else
+                {
+                    urlList += list.first().url;
+                }
+            }
         }
 
-        if (isFirstLoad && !isRekonqCrashed)
+        // first argument: 99% of the time we have just that...
+        if (isFirstLoad)
         {
             // No windows in the current desktop? No windows at all?
             // Create a new one and load there sites...
@@ -200,26 +243,35 @@ int Application::newInstance()
         }
         else
         {
-            if (ReKonfig::openTabNoWindow())
+            if (!ReKonfig::openExternalLinksInNewWindow())
             {
-                loadUrl(urlList.at(0), Rekonq::NewTab);
-                if (!mainWindow()->isActiveWindow())
-                    KWindowSystem::demandAttention(mainWindow()->winId(), true);
+                loadUrl(urlList.at(0), Rekonq::NewFocusedTab);
             }
             else
+            {
                 loadUrl(urlList.at(0), Rekonq::NewWindow);
+            }
+
+            if (!mainWindow()->isActiveWindow())
+                KWindowSystem::demandAttention(mainWindow()->winId(), true);
         }
 
-        for (int i = 1; i < urlList.count(); ++i)
-            loadUrl(urlList.at(i), Rekonq::NewTab);
-
-        KStartupInfo::appStarted();
-
+        // following arguments: what's best behavior here? I'm pretty sure no one has real opinion here
+        if (!ReKonfig::openExternalLinksInNewWindow())
+        {
+            for (int i = 1; i < urlList.count(); ++i)
+                loadUrl(urlList.at(i), Rekonq::NewFocusedTab);
+        }
+        else
+        {
+            for (int i = 1; i < urlList.count(); ++i)
+                loadUrl(urlList.at(i), Rekonq::NewWindow);
+        }
     }
     else
     {
-
-        if (isFirstLoad && !isRekonqCrashed)  // we are starting rekonq, for the first time with no args: use startup behaviour
+        // we are starting rekonq, for the first time with no args: use startup behaviour
+        if (isFirstLoad && !isRekonqCrashed)
         {
             switch (ReKonfig::startupBehaviour())
             {
@@ -273,18 +325,16 @@ int Application::newInstance()
         historyManager();
 
         // bookmarks loading
-        connect(bookmarkProvider(), SIGNAL(openUrl(const KUrl&, const Rekonq::OpenType&)),
-                instance(), SLOT(loadUrl(const KUrl&, const Rekonq::OpenType&)));
+        connect(bookmarkManager(), SIGNAL(openUrl(KUrl,Rekonq::OpenType)),
+                instance(), SLOT(loadUrl(KUrl,Rekonq::OpenType)));
 
-        // crash recovering
-        if (ReKonfig::recoverOnCrash())
-        {
-            mainWindow()->currentTab()->showMessageBar();
-        }
         ReKonfig::setRecoverOnCrash(ReKonfig::recoverOnCrash() + 1);
         saveConfiguration();
     }
 
+    KStartupInfo::appStarted();
+    isFirstLoad = false;
+
     return exitValue;
 }
 
@@ -331,13 +381,13 @@ HistoryManager *Application::historyManager()
 }
 
 
-BookmarkProvider *Application::bookmarkProvider()
+BookmarkManager *Application::bookmarkManager()
 {
-    if (m_bookmarkProvider.isNull())
+    if (m_bookmarkManager.isNull())
     {
-        m_bookmarkProvider = new BookmarkProvider;
+        m_bookmarkManager = new BookmarkManager;
     }
-    return m_bookmarkProvider.data();
+    return m_bookmarkManager.data();
 }
 
 
@@ -381,6 +431,26 @@ DownloadManager *Application::downloadManager()
 }
 
 
+UserAgentManager *Application::userAgentManager()
+{
+    if (m_userAgentManager.isNull())
+    {
+        m_userAgentManager = new UserAgentManager(instance());
+    }
+    return m_userAgentManager.data();
+}
+
+
+SyncManager *Application::syncManager()
+{
+    if (m_syncManager.isNull())
+    {
+        m_syncManager = new SyncManager(instance());
+    }
+    return m_syncManager.data();
+}
+
+
 void Application::loadUrl(const KUrl& url, const Rekonq::OpenType& type)
 {
     if (url.isEmpty())
@@ -392,23 +462,34 @@ void Application::loadUrl(const KUrl& url, const Rekonq::OpenType& type)
         return;
     }
 
+    Rekonq::OpenType newType = type;
+    // Don't open useless tabs or windows for actions in about: pages
+    if (url.url().contains("about:") && url.url().contains("/"))
+        newType = Rekonq::CurrentTab;
+
     // first, create the webview(s) to not let hangs UI..
     WebTab *tab = 0;
     MainWindow *w = 0;
-    w = (type == Rekonq::NewWindow)
+    w = (newType == Rekonq::NewWindow)
         ? newMainWindow()
         : mainWindow();
 
-    switch (type)
+    // be SURE window exists
+    if (!w)
+        w = newMainWindow();
+
+    switch (newType)
     {
     case Rekonq::NewTab:
-        if (ReKonfig::openTabNoWindow())
-            tab = w->mainView()->newWebTab(!ReKonfig::openTabsBack());
-        else
+        if (ReKonfig::openLinksInNewWindow())
         {
             w = newMainWindow();
             tab = w->mainView()->currentWebTab();
         }
+        else
+        {
+            tab = w->mainView()->newWebTab(!ReKonfig::openNewTabsInBackground());
+        }
         break;
     case Rekonq::NewFocusedTab:
         tab = w->mainView()->newWebTab(true);
@@ -572,15 +653,14 @@ void Application::updateConfiguration()
     defaultSettings->setDefaultTextEncoding(enc);
 
     // ================ WebKit ============================
-    defaultSettings->setAttribute(QWebSettings::AutoLoadImages, ReKonfig::autoLoadImages());
     defaultSettings->setAttribute(QWebSettings::DnsPrefetchEnabled, ReKonfig::dnsPrefetch());
+    defaultSettings->setAttribute(QWebSettings::PrintElementBackgrounds, ReKonfig::printElementBackgrounds());
+
     defaultSettings->setAttribute(QWebSettings::JavascriptEnabled, ReKonfig::javascriptEnabled());
-    defaultSettings->setAttribute(QWebSettings::JavaEnabled, ReKonfig::javaEnabled());
     defaultSettings->setAttribute(QWebSettings::JavascriptCanOpenWindows, ReKonfig::javascriptCanOpenWindows());
     defaultSettings->setAttribute(QWebSettings::JavascriptCanAccessClipboard, ReKonfig::javascriptCanAccessClipboard());
-    defaultSettings->setAttribute(QWebSettings::LinksIncludedInFocusChain, ReKonfig::linksIncludedInFocusChain());
-    defaultSettings->setAttribute(QWebSettings::ZoomTextOnly, ReKonfig::zoomTextOnly());
-    defaultSettings->setAttribute(QWebSettings::PrintElementBackgrounds, ReKonfig::printElementBackgrounds());
+
+    defaultSettings->setAttribute(QWebSettings::JavaEnabled, ReKonfig::javaEnabled());
 
     if (ReKonfig::pluginsEnabled() == 2)
         defaultSettings->setAttribute(QWebSettings::PluginsEnabled, false);
@@ -602,11 +682,12 @@ void Application::updateConfiguration()
         QWebSettings::setOfflineStorageDefaultQuota(50000);
     }
 
-    // Applies user/system defined CSS to all open webpages.
-    ReKonfig::userCSS().isEmpty()
-    ? defaultSettings->setUserStyleSheetUrl(KUrl(KStandardDirs::locate("appdata" , "default_rekonq.css")))
-    : defaultSettings->setUserStyleSheetUrl(ReKonfig::userCSS())
-    ;
+    // ================= WebGl ===================
+    defaultSettings->setAttribute(QWebSettings::WebGLEnabled, ReKonfig::webGL());
+    defaultSettings->setAttribute(QWebSettings::AcceleratedCompositingEnabled, ReKonfig::webGL());
+
+    // Applies user defined CSS to all open webpages.
+    defaultSettings->setUserStyleSheetUrl(ReKonfig::userCSS());
 
     // ====== load Settings on main classes
     historyManager()->loadSettings();
@@ -616,10 +697,10 @@ void Application::updateConfiguration()
     if (!mainWindow())
         return;
 
-    if (!ReKonfig::useFavicon())
-        mainWindow()->setWindowIcon(KIcon("rekonq"));
-    else
-        mainWindow()->changeWindowIcon(mainWindow()->mainView()->currentIndex());
+    ReKonfig::useFavicon()
+    ? mainWindow()->changeWindowIcon(mainWindow()->mainView()->currentIndex())
+    : mainWindow()->setWindowIcon(KIcon("rekonq"))
+    ;
 
     // hovering unfocused tabs options
     switch (ReKonfig::hoveringTabOption())
@@ -628,7 +709,7 @@ void Application::updateConfiguration()
     case 3: // nothing
         for (int i = 0; i < mainWindow()->mainView()->tabBar()->count(); i++)
         {
-            mainWindow()->mainView()->tabBar()->setTabToolTip(i, "");
+            mainWindow()->mainView()->tabBar()->setTabToolTip(i, QL1S(""));
         }
         break;
 
@@ -728,7 +809,7 @@ void Application::queryQuit()
     {
         int answer = KMessageBox::questionYesNoCancel(
                          mainWindow(),
-                         i18n("Wanna close the window or the whole app?"),
+                         i18n("Do you want to close the window or the whole application?"),
                          i18n("Application/Window closing..."),
                          KGuiItem(i18n("C&lose Current Window"),
                                   KIcon("window-close")),
@@ -754,3 +835,85 @@ void Application::queryQuit()
     // in case of just one window...
     quit();
 }
+
+
+void Application::createWebAppShortcut()
+{
+    KUrl u = mainWindow()->currentTab()->url();
+    QString h = u.host();
+
+    QPointer<KDialog> dialog = new KDialog(mainWindow());
+    dialog->setCaption(i18nc("@title:window", "Create Application Shortcut"));
+    dialog->setButtons(KDialog::Ok | KDialog::Cancel);
+    dialog->button(KDialog::Ok)->setText(i18n("Create"));
+
+    Ui::webAppCreation wAppWidget;
+    QWidget widget;
+    wAppWidget.setupUi(&widget);
+
+    const QString title = mainWindow()->currentTab()->view()->title().remove('&');
+    wAppWidget.iconLabel->setPixmap(iconManager()->iconForUrl(u).pixmap(32));
+    wAppWidget.titleLabel->setText(title);
+    wAppWidget.kcfg_createDesktopAppShortcut->setChecked(ReKonfig::createDesktopAppShortcut());
+    wAppWidget.kcfg_createMenuAppShortcut->setChecked(ReKonfig::createMenuAppShortcut());
+
+    dialog->setMainWidget(&widget);
+    dialog->exec();
+
+    if (dialog->result() == QDialog::Accepted)
+    {
+        ReKonfig::setCreateDesktopAppShortcut(wAppWidget.kcfg_createDesktopAppShortcut->isChecked());
+        ReKonfig::setCreateMenuAppShortcut(wAppWidget.kcfg_createMenuAppShortcut->isChecked());
+
+        iconManager()->saveDesktopIconForUrl(u);
+        QString iconPath = KStandardDirs::locateLocal("cache" , "favicons/" , true) + h + QL1S("_WEBAPPICON.png");
+
+        QString shortcutString = QL1S("#!/usr/bin/env xdg-open\n")
+                                 + QL1S("[Desktop Entry]\n")
+                                 + QL1S("name=kwebapp\n")
+                                 + QL1S("Icon=") + iconPath + QL1S("\n")
+                                 + QL1S("Exec=kwebapp ") + u.url() + QL1S("\n")
+                                 + QL1S("Type=Application\n")
+                                 + QL1S("Categories=Application;Network;WebBrowser\n")
+                                 ;
+
+        if (ReKonfig::createDesktopAppShortcut())
+        {
+            QString desktop = KGlobalSettings::desktopPath();
+            QFile wAppFile(desktop + QL1C('/') + title + QL1S(".desktop"));
+
+            if (!wAppFile.open(QIODevice::WriteOnly | QIODevice::Text))
+            {
+                kDebug() << "Unable to open file: " << wAppFile.errorString();
+                return;
+            }
+
+            QTextStream out(&wAppFile);
+            out.setCodec("UTF-8");
+            out << shortcutString;
+
+            wAppFile.close();
+        }
+
+        if (ReKonfig::createMenuAppShortcut())
+        {
+            QString appMenuDir = KStandardDirs::locateLocal("xdgdata-apps", QString());
+            QFile wAppFile(appMenuDir + QL1C('/') + title + QL1S(".desktop"));
+
+            if (!wAppFile.open(QIODevice::WriteOnly | QIODevice::Text))
+            {
+                kDebug() << "Unable to open file: " << wAppFile.errorString();
+                return;
+            }
+
+            QTextStream out(&wAppFile);
+            out.setCodec("UTF-8");
+            out << shortcutString;
+
+            wAppFile.close();
+        }
+
+    }
+
+    dialog->deleteLater();
+}
