@@ -2,7 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2012 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2009 by Paweł Prażak <pawelprazak at gmail dot com>
 * Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
 * Copyright (C) 2010 by Matthieu Gicquel <matgic78 at gmail dot com>
@@ -56,8 +56,6 @@
 #include <QtGui/QMovie>
 #include <QtGui/QToolButton>
 
-//Hack:
-QString temporaryUglyHackString = "";
 
 MainView::MainView(QWidget *parent)
     : KTabWidget(parent)
@@ -89,7 +87,7 @@ MainView::MainView(QWidget *parent)
     connect(tabBar, SIGNAL(detachTab(int)),         this,   SLOT(detachTab(int)));
 
     connect(tabBar, SIGNAL(tabCloseRequested(int)), this,   SLOT(closeTab(int)));
-    connect(tabBar, SIGNAL(tabMoved(int, int)),     m_widgetBar, SLOT(moveBar(int, int)));
+    connect(tabBar, SIGNAL(tabMoved(int,int)),     m_widgetBar, SLOT(moveBar(int,int)));
 
     // current page index changing
     connect(this, SIGNAL(currentChanged(int)), this, SLOT(currentChanged(int)));
@@ -144,17 +142,24 @@ void MainView::updateTabBar()
 {
     if (ReKonfig::alwaysShowTabBar() || count() > 1)
     {
-        if (tabBar()->isHidden())
+        // Get sure tabbar is well shown (and hided) during fullscreen navigation
+        // NOTE: don't ask me why, but it seems that using code like:
+        // MainWindow *w = qobject_cast<MainWindow *>(parent());
+        // does NOT work here. So, I'm asking you: WHY???
+        MainWindow *w = rApp->mainWindow();
+        if (w && !w->isFullScreen())
         {
-            tabBar()->show();
-        }
+            if (tabBar()->isHidden())
+            {
+                tabBar()->show();
+            }
 
-        // this to ensure tab button visibility also on new window creation
-        if (m_addTabButton->isHidden())
-        {
-            m_addTabButton->show();
+            // this to ensure tab button visibility also on new window creation
+            if (m_addTabButton->isHidden())
+            {
+                m_addTabButton->show();
+            }
         }
-
     }
     else
     {
@@ -241,16 +246,16 @@ void MainView::currentChanged(int index)
     if (oldTab)
     {
         // disconnecting webpage from mainview
-        disconnect(oldTab->page(), SIGNAL(statusBarMessage(const QString&)),
-                   this, SIGNAL(showStatusBarMessage(const QString&)));
-        disconnect(oldTab->page(), SIGNAL(linkHovered(const QString&, const QString&, const QString&)),
-                   this, SIGNAL(linkHovered(const QString&)));
+        disconnect(oldTab->page(), SIGNAL(statusBarMessage(QString)),
+                   this, SIGNAL(showStatusBarMessage(QString)));
+        disconnect(oldTab->page(), SIGNAL(linkHovered(QString,QString,QString)),
+                   this, SIGNAL(linkHovered(QString)));
     }
 
-    connect(tab->page(), SIGNAL(statusBarMessage(const QString&)),
-            this, SIGNAL(showStatusBarMessage(const QString&)));
-    connect(tab->page(), SIGNAL(linkHovered(const QString&, const QString&, const QString&)),
-            this, SIGNAL(linkHovered(const QString&)));
+    connect(tab->page(), SIGNAL(statusBarMessage(QString)),
+            this, SIGNAL(showStatusBarMessage(QString)));
+    connect(tab->page(), SIGNAL(linkHovered(QString,QString,QString)),
+            this, SIGNAL(linkHovered(QString)));
 
     emit currentTitle(tab->view()->title());
     m_widgetBar->setCurrentIndex(index);
@@ -259,7 +264,7 @@ void MainView::currentChanged(int index)
     emit showStatusBarMessage(QString());
 
     // notify UI to eventually switch stop/reload button
-    emit browserTabLoading(tab->isPageLoading());
+    emit currentTabStateChanged();
 
     // set focus to the current webview
     if (tab->url().scheme() == QL1S("about"))
@@ -291,26 +296,27 @@ WebTab *MainView::newWebTab(bool focused)
     // connecting webview with mainview
     connect(tab->view(), SIGNAL(loadStarted()), this, SLOT(webViewLoadStarted()));
     connect(tab->view(), SIGNAL(loadFinished(bool)), this, SLOT(webViewLoadFinished(bool)));
-    connect(tab, SIGNAL(titleChanged(const QString &)), this, SLOT(webViewTitleChanged(const QString &)));
-    connect(tab->view(), SIGNAL(urlChanged(const QUrl &)), this, SLOT(webViewUrlChanged(const QUrl &)));
+    connect(tab, SIGNAL(titleChanged(QString)), this, SLOT(webViewTitleChanged(QString)));
+    connect(tab->view(), SIGNAL(urlChanged(QUrl)), this, SLOT(webViewUrlChanged(QUrl)));
     connect(tab->view(), SIGNAL(iconChanged()), this, SLOT(webViewIconChanged()));
     connect(tab->view(), SIGNAL(openPreviousInHistory()), this, SIGNAL(openPreviousInHistory()));
     connect(tab->view(), SIGNAL(openNextInHistory()), this, SIGNAL(openNextInHistory()));
 
     // connecting webPage signals with mainview
     connect(tab->page(), SIGNAL(windowCloseRequested()), this, SLOT(windowCloseRequested()));
-    connect(tab->page(), SIGNAL(printRequested(QWebFrame *)), this, SIGNAL(printRequested(QWebFrame *)));
+    connect(tab->page(), SIGNAL(printRequested(QWebFrame*)), this, SIGNAL(printRequested(QWebFrame*)));
 
-    if (ReKonfig::openTabsNearCurrent())
+    bool openNearCurrent = true;
+    if (sender())
+        openNearCurrent = sender()->objectName() != "new_tab" ? true : false;
+    if (ReKonfig::openNewTabsNearCurrent() && openNearCurrent)
     {
         insertTab(currentIndex() + 1, tab, i18n("(Untitled)"));
-        temporaryUglyHackString = tabText(currentIndex() + 1);
         m_widgetBar->insertWidget(currentIndex() + 1, tab->urlBar());
     }
     else
     {
         addTab(tab, i18n("(Untitled)"));
-        temporaryUglyHackString = tabText(count() - 1);
         m_widgetBar->addWidget(tab->urlBar());
     }
     updateTabBar();
@@ -359,6 +365,9 @@ void MainView::reloadAllTabs()
 void MainView::windowCloseRequested()
 {
     WebPage *page = qobject_cast<WebPage *>(sender());
+    if (!page)
+        return;
+
     WebView *view = qobject_cast<WebView *>(page->view());
     int index = indexOf(view->parentWidget());
 
@@ -418,14 +427,45 @@ void MainView::closeTab(int index, bool del)
     if (index < 0 || index >= count())
         return;
 
+    WebTab *tabToClose = webTab(index);
+    if (!tabToClose)
+        return;
+
+    if (tabToClose->view()->isModified())
+    {
+        int risp = KMessageBox::warningContinueCancel(this,
+                   i18n("This tab contains changes that have not been submitted.\n"
+                        "Closing the tab will discard these changes.\n"
+                        "Do you really want to close this tab?\n"),
+                   i18n("Closing Modified Tab"), KGuiItem(i18n("Close &Tab"), "tab-close"), KStandardGuiItem::cancel());
+        if (risp != KMessageBox::Continue)
+            return;
+    }
+
+    kDebug() << "URL: " << tabToClose->url();
+    kDebug() << "rekonq page: " << tabToClose->page()->isOnRekonqPage();
+
+    if (!tabToClose->url().isEmpty()
+            && tabToClose->url().scheme() != QL1S("about")
+            && !tabToClose->page()->isOnRekonqPage()
+            && !QWebSettings::globalSettings()->testAttribute(QWebSettings::PrivateBrowsingEnabled)
+       )
+    {
+        const int recentlyClosedTabsLimit = 8;
+        TabHistory history(tabToClose->view()->history());
+
+        m_recentlyClosedTabs.removeAll(history);
+        if (m_recentlyClosedTabs.count() == recentlyClosedTabsLimit)
+            m_recentlyClosedTabs.removeLast();
+        m_recentlyClosedTabs.prepend(history);
+    }
+
+    // what to do if there is just one tab...
     if (count() == 1)
     {
         if (ReKonfig::lastTabClosesWindow())
         {
             emit closeWindow();
-//             // closing window...
-//             MainWindow *w = qobject_cast<MainWindow *>(parent());
-//             w->close();
             return;
         }
 
@@ -450,34 +490,7 @@ void MainView::closeTab(int index, bool del)
         }
         return;
     }
-
-    WebTab *tabToClose = webTab(index);
-    if (!tabToClose)
-        return;
-
-    if (tabToClose->view()->isModified())
-    {
-        int risp = KMessageBox::warningContinueCancel(this,
-                   i18n("This tab contains changes that have not been submitted.\n"
-                        "Closing the tab will discard these changes.\n"
-                        "Do you really want to close this tab?\n"),
-                   i18n("Closing Modified Tab"), KGuiItem(i18n("Close &Tab"), "tab-close"), KStandardGuiItem::cancel());
-        if (risp != KMessageBox::Continue)
-            return;
-    }
-
-    if (!tabToClose->url().isEmpty()
-            && !QWebSettings::globalSettings()->testAttribute(QWebSettings::PrivateBrowsingEnabled)
-       )
-    {
-        const int recentlyClosedTabsLimit = 10;
-        TabHistory history(tabToClose->view()->history());
-
-        m_recentlyClosedTabs.removeAll(history);
-        if (m_recentlyClosedTabs.count() == recentlyClosedTabsLimit)
-            m_recentlyClosedTabs.removeLast();
-        m_recentlyClosedTabs.prepend(history);
-    }
+    // else...
 
     removeTab(index);
     updateTabBar();        // UI operation: do it ASAP!!
@@ -491,9 +504,13 @@ void MainView::closeTab(int index, bool del)
     }
 }
 
+
 void MainView::webViewLoadStarted()
 {
     WebView *view = qobject_cast<WebView *>(sender());
+    if (!view)
+        return;
+
     int index = indexOf(view->parentWidget());
     if (-1 != index)
     {
@@ -507,10 +524,12 @@ void MainView::webViewLoadStarted()
     if (index != currentIndex())
         return;
 
-    emit browserTabLoading(true);
+    emit currentTabStateChanged();
     emit showStatusBarMessage(i18n("Loading..."), Rekonq::Info);
 
-    if (view == currentWebTab()->view() && view->url().scheme() != QL1S("about"))
+    if (view == currentWebTab()->view()
+            && !currentUrlBar()->hasFocus()
+            && view->url().scheme() != QL1S("about"))
     {
         view->setFocus();
     }
@@ -533,7 +552,7 @@ void MainView::webViewLoadFinished(bool ok)
     }
 
     webViewIconChanged();
-    emit browserTabLoading(false);
+    emit currentTabStateChanged();
 
     // don't display messages for background tabs
     if (index != currentIndex())
@@ -551,8 +570,11 @@ void MainView::webViewLoadFinished(bool ok)
 void MainView::webViewIconChanged()
 {
     WebView *view = qobject_cast<WebView *>(sender());
+    if (!view)
+        return;
+
     WebTab *tab = qobject_cast<WebTab *>(view->parent());
-    int index = indexOf(tab);
+    const int index = indexOf(tab);
 
     if (-1 != index)
     {
@@ -573,20 +595,25 @@ void MainView::webViewTitleChanged(const QString &title)
     tabTitle.replace('&', "&&");
 
     WebTab *tab = qobject_cast<WebTab *>(sender());
+    if (!tab)
+        return;
+
     int index = indexOf(tab);
     if (-1 != index)
     {
         setTabText(index, tabTitle);
     }
+
     if (currentIndex() == index)
     {
         emit currentTitle(viewTitle);
     }
     else
     {
-        if (tabTitle != temporaryUglyHackString)
+        if (tabTitle != i18n("(Untitled)"))
             tabBar()->setTabHighlighted(index);
     }
+
     rApp->historyManager()->updateHistoryEntry(tab->url(), tabTitle);
     if (ReKonfig::hoveringTabOption() == 1)
         tabBar()->setTabToolTip(index, tabTitle.remove('&'));
@@ -596,13 +623,14 @@ void MainView::webViewTitleChanged(const QString &title)
 void MainView::webViewUrlChanged(const QUrl &url)
 {
     WebView *view = qobject_cast<WebView *>(sender());
+    if (!view)
+        return;
+
     int index = indexOf(view->parentWidget());
-    if (-1 != index)
-    {
-        tabBar()->setTabData(index, url);
-    }
     if (ReKonfig::hoveringTabOption() == 2)
-        tabBar()->setTabToolTip(index, webTab(index)->url().toMimeDataString());
+        tabBar()->setTabToolTip(index, url.toString());
+    
+    rApp->mainWindow()->updateHistoryActions();
 }
 
 
@@ -624,37 +652,34 @@ void MainView::previousTab()
 }
 
 
-void MainView::openLastClosedTab()
+void MainView::openClosedTab()
+{
+    KAction *action = qobject_cast<KAction *>(sender());
+    if (!action)
+        return;
+
+    int index = action->data().toInt();
+    kDebug() << "TAB INDEX TO RESTORE:" << index;
+    restoreClosedTab(index);
+}
+
+
+void MainView::restoreClosedTab(int i, bool inNewTab)
 {
     if (m_recentlyClosedTabs.isEmpty())
         return;
 
-    TabHistory history = m_recentlyClosedTabs.takeFirst();
-    WebView *view = rApp->mainWindow()->mainView()->newWebTab()->view();
+    TabHistory history = m_recentlyClosedTabs.takeAt(i);
+
+    WebView *view = inNewTab
+                    ? newWebTab()->view()
+                    : currentWebTab()->view()
+                    ;
 
     history.applyHistory(view->history());
-}
 
-
-void MainView::openClosedTab()
-{
-    KAction *action = qobject_cast<KAction *>(sender());
-    if (action)
-    {
-        WebView *view = rApp->mainWindow()->mainView()->newWebTab()->view();
-        TabHistory history;
-        Q_FOREACH(const TabHistory & item, m_recentlyClosedTabs)
-        {
-            if (item.history == action->data().toByteArray())
-            {
-                history = item;
-                break;
-            }
-        }
-        history.applyHistory(view->history());
-
-        m_recentlyClosedTabs.removeAll(history);
-    }
+    // just to get sure...
+    m_recentlyClosedTabs.removeAll(history);
 }
 
 
@@ -732,8 +757,38 @@ void MainView::detachTab(int index, MainWindow *toWindow)
             w = rApp->newMainWindow(false);
         else
             w = toWindow;
-        w->mainView()->addTab(tab, rApp->iconManager()->iconForUrl(u), label);
+
+        w->mainView()->addTab(tab, label);
         w->mainView()->widgetBar()->insertWidget(0, bar);
         w->mainView()->updateTabBar();
+
+        // reconnect signals to the new mainview
+        // Code copied from newWebTab(), any new changes there should be applied here
+
+        // disconnecting webview with old mainview
+        disconnect(tab->view(), SIGNAL(loadStarted()));
+        disconnect(tab->view(), SIGNAL(loadFinished(bool)));
+        disconnect(tab, SIGNAL(titleChanged(QString)));
+        disconnect(tab->view(), SIGNAL(urlChanged(QUrl)));
+        disconnect(tab->view(), SIGNAL(iconChanged()));
+        disconnect(tab->view(), SIGNAL(openPreviousInHistory()));
+        disconnect(tab->view(), SIGNAL(openNextInHistory()));
+
+        // disconnecting webPage signals with old mainview
+        disconnect(tab->page(), SIGNAL(windowCloseRequested()));
+        disconnect(tab->page(), SIGNAL(printRequested(QWebFrame*)));
+
+        // connecting webview with new mainview
+        connect(tab->view(), SIGNAL(loadStarted()), w->mainView(), SLOT(webViewLoadStarted()));
+        connect(tab->view(), SIGNAL(loadFinished(bool)), w->mainView(), SLOT(webViewLoadFinished(bool)));
+        connect(tab, SIGNAL(titleChanged(QString)), w->mainView(), SLOT(webViewTitleChanged(QString)));
+        connect(tab->view(), SIGNAL(urlChanged(QUrl)), w->mainView(), SLOT(webViewUrlChanged(QUrl)));
+        connect(tab->view(), SIGNAL(iconChanged()), w->mainView(), SLOT(webViewIconChanged()));
+        connect(tab->view(), SIGNAL(openPreviousInHistory()), w->mainView(), SIGNAL(openPreviousInHistory()));
+        connect(tab->view(), SIGNAL(openNextInHistory()), w->mainView(), SIGNAL(openNextInHistory()));
+
+        // connecting webPage signals with new mainview
+        connect(tab->page(), SIGNAL(windowCloseRequested()), w->mainView(), SLOT(windowCloseRequested()));
+        connect(tab->page(), SIGNAL(printRequested(QWebFrame*)), w->mainView(), SIGNAL(printRequested(QWebFrame*)));
     }
 }
