@@ -2,7 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2012 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2009 by Paweł Prażak <pawelprazak at gmail dot com>
 * Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
 * Copyright (C) 2010 by Matthieu Gicquel <matgic78 at gmail dot com>
@@ -38,7 +38,7 @@
 #include "adblockmanager.h"
 #include "analyzerpanel.h"
 #include "application.h"
-#include "bookmarkprovider.h"
+#include "bookmarkmanager.h"
 #include "bookmarkspanel.h"
 #include "bookmarkstoolbar.h"
 #include "downloadmanager.h"
@@ -46,17 +46,18 @@
 #include "historypanel.h"
 #include "iconmanager.h"
 #include "mainview.h"
+#include "rekonqmenu.h"
 #include "sessionmanager.h"
 #include "settingsdialog.h"
 #include "stackedurlbar.h"
+#include "syncmanager.h"
 #include "tabbar.h"
 #include "urlbar.h"
 #include "webinspectorpanel.h"
 #include "webpage.h"
 #include "webtab.h"
 #include "zoombar.h"
-#include "useragentinfo.h"
-#include "useragentwidget.h"
+#include "useragentmanager.h"
 
 // Ui Includes
 #include "ui_cleardata.h"
@@ -78,6 +79,7 @@
 #include <KTemporaryFile>
 #include <KToggleFullScreenAction>
 #include <KXMLGUIFactory>
+#include <kdeprintdialog.h>
 
 #include <KParts/Part>
 #include <KParts/BrowserExtension>
@@ -93,7 +95,6 @@
 #include <QtGui/QLabel>
 #include <QtGui/QPrintDialog>
 #include <QtGui/QPrinter>
-#include <QtGui/QPrintPreviewDialog>
 #include <QtGui/QVBoxLayout>
 
 #include <QtWebKit/QWebHistory>
@@ -111,9 +112,10 @@ MainWindow::MainWindow()
     , m_bookmarksPanel(0)
     , m_webInspectorPanel(0)
     , m_analyzerPanel(0)
+    , m_loadStopReloadAction(0)
     , m_historyBackMenu(0)
     , m_historyForwardMenu(0)
-    , m_userAgentMenu(new KMenu(this))
+    , m_tabListMenu(0)
     , m_bookmarksBar(0)
     , m_popup(new QLabel(this))
     , m_hidePopupTimer(new QTimer(this))
@@ -171,7 +173,7 @@ MainWindow::MainWindow()
     setupBookmarksAndToolsShortcuts();
 
     // setting popup notification
-    connect(rApp, SIGNAL(focusChanged(QWidget*, QWidget*)), m_popup, SLOT(hide()));
+    connect(rApp, SIGNAL(focusChanged(QWidget*,QWidget*)), m_popup, SLOT(hide()));
     m_popup->setAutoFillBackground(true);
     m_popup->setMargin(4);
     m_popup->raise();
@@ -179,12 +181,14 @@ MainWindow::MainWindow()
     connect(m_hidePopupTimer, SIGNAL(timeout()), m_popup, SLOT(hide()));
 
     // notification system
-    connect(m_view, SIGNAL(showStatusBarMessage(const QString&, Rekonq::Notify)), this, SLOT(notifyMessage(const QString&, Rekonq::Notify)));
-    connect(m_view, SIGNAL(linkHovered(const QString&)), this, SLOT(notifyMessage(const QString&)));
+    connect(m_view, SIGNAL(showStatusBarMessage(QString,Rekonq::Notify)), this, SLOT(notifyMessage(QString,Rekonq::Notify)));
+    connect(m_view, SIGNAL(linkHovered(QString)), this, SLOT(notifyMessage(QString)));
+    connect(rApp->downloadManager(), SIGNAL(notifyDownload(QString,Rekonq::Notify)),
+            this, SLOT(notifyMessage(QString,Rekonq::Notify)));
 
     // connect signals and slots
-    connect(m_view, SIGNAL(currentTitle(const QString &)), this, SLOT(updateWindowTitle(const QString &)));
-    connect(m_view, SIGNAL(printRequested(QWebFrame *)), this, SLOT(printRequested(QWebFrame *)));
+    connect(m_view, SIGNAL(currentTitle(QString)), this, SLOT(updateWindowTitle(QString)));
+    connect(m_view, SIGNAL(printRequested(QWebFrame*)), this, SLOT(printRequested(QWebFrame*)));
     connect(m_view, SIGNAL(closeWindow()), this, SLOT(close()));
 
     // (shift +) ctrl + tab switching
@@ -195,14 +199,15 @@ MainWindow::MainWindow()
     connect(m_view, SIGNAL(openPreviousInHistory()), this, SLOT(openPrevious()));
     connect(m_view, SIGNAL(openNextInHistory()), this, SLOT(openNext()));
 
-    // update toolbar actions signals
-    connect(m_view, SIGNAL(currentChanged(int)), this, SLOT(updateActions()));
+    // update actions
+    connect(m_view, SIGNAL(currentChanged(int)), this, SLOT(updateHistoryActions()));
+    connect(m_view, SIGNAL(currentTabStateChanged()), this, SLOT(updateTabActions()));
 
     // Change window icon according to tab icon
     connect(m_view, SIGNAL(currentChanged(int)), this, SLOT(changeWindowIcon(int)));
 
     // Find Bar signal
-    connect(m_findBar, SIGNAL(searchString(const QString &)), this, SLOT(find(const QString &)));
+    connect(m_findBar, SIGNAL(searchString(QString)), this, SLOT(find(QString)));
 
     // Zoom Bar signal
     connect(m_view, SIGNAL(currentChanged(int)), m_zoomBar, SLOT(updateSlider(int)));
@@ -216,8 +221,8 @@ MainWindow::MainWindow()
     // accept d'n'd
     setAcceptDrops(true);
 
-    // Bookmark ToolBar (needs to be setup after the call to setupGUI())
-    initBookmarkBar();
+    // Things that need to be setup after the call to setupGUI() and after ctor call
+    QTimer::singleShot(1, this, SLOT(postLaunch()));
 }
 
 
@@ -225,8 +230,8 @@ MainWindow::~MainWindow()
 {
     m_hidePopupTimer->stop();
 
-    rApp->bookmarkProvider()->removeBookmarkBar(m_bookmarksBar);
-    rApp->bookmarkProvider()->removeBookmarkPanel(m_bookmarksPanel);
+    rApp->bookmarkManager()->removeBookmarkBar(m_bookmarksBar);
+    rApp->bookmarkManager()->removeBookmarkPanel(m_bookmarksPanel);
     rApp->removeMainWindow(this);
 }
 
@@ -239,29 +244,42 @@ void MainWindow::setupToolbars()
     a = new KAction(i18n("Location Bar"), this);
     a->setDefaultWidget(m_view->widgetBar());
     actionCollection()->addAction(QL1S("url_bar"), a);
-
-    KToolBar *mainBar = toolBar("mainToolBar");
-
-    mainBar->show();  // this just to fix reopening rekonq after fullscreen close
 }
 
 
-void MainWindow::initBookmarkBar()
+void MainWindow::postLaunch()
 {
+    // this just to fix reopening rekonq after fullscreen close
+    KToolBar *mainBar = toolBar("mainToolBar");
+    mainBar->show();  
+    
+    // Bookmarks Bar
     KToolBar *XMLGUIBkBar = toolBar("bookmarkToolBar");
     if (!XMLGUIBkBar)
         return;
 
     if (m_bookmarksBar)
     {
-        rApp->bookmarkProvider()->removeBookmarkBar(m_bookmarksBar);
+        rApp->bookmarkManager()->removeBookmarkBar(m_bookmarksBar);
         delete m_bookmarksBar;
     }
     m_bookmarksBar = new BookmarkToolBar(XMLGUIBkBar, this);
-    rApp->bookmarkProvider()->registerBookmarkBar(m_bookmarksBar);
-    QAction *a = actionByName(QL1S("show_bookmarks_toolbar"));
+    rApp->bookmarkManager()->registerBookmarkBar(m_bookmarksBar);
+
+    QAction *a;
+
+    // Bookmarks bar action
+    a = actionByName(QL1S("show_bookmarks_toolbar"));
     a->setChecked(XMLGUIBkBar->isVisible());
-    connect(a, SIGNAL(toggled(bool)), this, SLOT(toggleBookmarkBarVisible(bool)));
+    connect(XMLGUIBkBar, SIGNAL(visibilityChanged(bool)), a, SLOT(setChecked(bool)));
+
+    // History panel action
+    a = actionByName(QL1S("show_history_panel"));
+    a->setChecked(m_historyPanel->isVisible());
+
+    // Bookmarks panel action
+    a = actionByName(QL1S("show_bookmarks_panel"));
+    a->setChecked(m_bookmarksPanel->isVisible());
 }
 
 
@@ -279,7 +297,7 @@ void MainWindow::configureToolbars()
 
     KEditToolBar dlg(factory(), this);
     // The bookmark bar needs to be refill after the UI changes are finished
-    connect(&dlg, SIGNAL(newToolBarConfig()), this, SLOT(initBookmarkBar()));
+    connect(&dlg, SIGNAL(newToolBarConfig()), this, SLOT(postLaunch()));
     dlg.exec();
 }
 
@@ -324,7 +342,6 @@ void MainWindow::setupActions()
 
     a = KStandardAction::find(m_findBar, SLOT(show()), actionCollection());
     KShortcut findShortcut = KStandardShortcut::find();
-    findShortcut.setAlternate(Qt::Key_Slash);
     a->setShortcut(findShortcut);
 
     KStandardAction::findNext(this, SLOT(findNext()) , actionCollection());
@@ -336,7 +353,7 @@ void MainWindow::setupActions()
     a->setShortcut(fullScreenShortcut);
 
     a = actionCollection()->addAction(KStandardAction::Home);
-    connect(a, SIGNAL(triggered(Qt::MouseButtons, Qt::KeyboardModifiers)), this, SLOT(homePage(Qt::MouseButtons, Qt::KeyboardModifiers)));
+    connect(a, SIGNAL(triggered(Qt::MouseButtons,Qt::KeyboardModifiers)), this, SLOT(homePage()));
     KStandardAction::preferences(this, SLOT(preferences()), actionCollection());
 
     a = KStandardAction::redisplay(m_view, SLOT(webReload()), actionCollection());
@@ -350,15 +367,15 @@ void MainWindow::setupActions()
     actionCollection()->addAction(QL1S("stop"), a);
     connect(a, SIGNAL(triggered(bool)), m_view, SLOT(webStop()));
 
-    // stop reload Action
-    m_stopReloadAction = new KAction(this);
-    actionCollection()->addAction(QL1S("stop_reload") , m_stopReloadAction);
-    m_stopReloadAction->setShortcutConfigurable(false);
-    connect(m_view, SIGNAL(browserTabLoading(bool)), this, SLOT(browserLoading(bool)));
+    // load stop reload Action
+    m_loadStopReloadAction = new KAction(this);
+    actionCollection()->addAction(QL1S("load_stop_reload") , m_loadStopReloadAction);
+    m_loadStopReloadAction->setShortcutConfigurable(false);
 
+    // Open location action
     a = new KAction(i18n("Open Location"), this);
     KShortcut openLocationShortcut(Qt::CTRL + Qt::Key_L);
-    openLocationShortcut.setAlternate(Qt::Key_F6);
+    openLocationShortcut.setAlternate(Qt::ALT + Qt::Key_D);
     a->setShortcut(openLocationShortcut);
     actionCollection()->addAction(QL1S("open_location"), a);
     connect(a, SIGNAL(triggered(bool)) , this, SLOT(openLocation()));
@@ -377,6 +394,12 @@ void MainWindow::setupActions()
     tabAction->setDelayed(false);
     actionCollection()->addAction(QL1S("tab_list"), tabAction);
 
+    //================================Download========================================
+    a = new KAction(KIcon("download"), i18n("Downloads"), this);
+    a->setShortcut(KShortcut(Qt::CTRL + Qt::Key_J));
+    actionCollection()->addAction(QL1S("openDownloadsPage"), a);
+    connect(a, SIGNAL(triggered(bool)), this, SLOT(openDownloadsPage()));
+
     // =============================== Tools Actions =================================
     a = new KAction(i18n("View Page S&ource"), this);
     a->setIcon(KIcon("application-xhtml+xml"));
@@ -395,22 +418,22 @@ void MainWindow::setupActions()
 
     // ========================= History related actions ==============================
     a = actionCollection()->addAction(KStandardAction::Back);
-    connect(a, SIGNAL(triggered(Qt::MouseButtons, Qt::KeyboardModifiers)),
-            this, SLOT(openPrevious(Qt::MouseButtons, Qt::KeyboardModifiers)));
+    connect(a, SIGNAL(triggered(Qt::MouseButtons,Qt::KeyboardModifiers)),
+            this, SLOT(openPrevious(Qt::MouseButtons,Qt::KeyboardModifiers)));
 
     m_historyBackMenu = new KMenu(this);
     a->setMenu(m_historyBackMenu);
     connect(m_historyBackMenu, SIGNAL(aboutToShow()), this, SLOT(aboutToShowBackMenu()));
-    connect(m_historyBackMenu, SIGNAL(triggered(QAction *)), this, SLOT(openActionUrl(QAction *)));
+    connect(m_historyBackMenu, SIGNAL(triggered(QAction*)), this, SLOT(openActionUrl(QAction*)));
 
     a = actionCollection()->addAction(KStandardAction::Forward);
-    connect(a, SIGNAL(triggered(Qt::MouseButtons, Qt::KeyboardModifiers)),
-            this, SLOT(openNext(Qt::MouseButtons, Qt::KeyboardModifiers)));
+    connect(a, SIGNAL(triggered(Qt::MouseButtons,Qt::KeyboardModifiers)),
+            this, SLOT(openNext(Qt::MouseButtons,Qt::KeyboardModifiers)));
 
     m_historyForwardMenu = new KMenu(this);
     a->setMenu(m_historyForwardMenu);
     connect(m_historyForwardMenu, SIGNAL(aboutToShow()), this, SLOT(aboutToShowForwardMenu()));
-    connect(m_historyForwardMenu, SIGNAL(triggered(QAction *)), this, SLOT(openActionUrl(QAction*)));
+    connect(m_historyForwardMenu, SIGNAL(triggered(QAction*)), this, SLOT(openActionUrl(QAction*)));
 
     // ============================== General Tab Actions ====================================
     a = new KAction(KIcon("tab-new"), i18n("New &Tab"), this);
@@ -433,17 +456,13 @@ void MainWindow::setupActions()
     connect(a, SIGNAL(triggered(bool)), m_view, SLOT(previousTab()));
 
     a = new KAction(KIcon("tab-new"), i18n("Open Last Closed Tab"), this);
+    a->setData(0);  // last tab has always index = 0
     a->setShortcut(KShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_T));
     actionCollection()->addAction(QL1S("open_last_closed_tab"), a);
-    connect(a, SIGNAL(triggered(bool)), m_view, SLOT(openLastClosedTab()));
+    connect(a, SIGNAL(triggered(bool)), m_view, SLOT(openClosedTab()));
 
-    // Closed Tabs Menu
-    KActionMenu *closedTabsMenu = new KActionMenu(KIcon("tab-new"), i18n("Closed Tabs"), this);
-    closedTabsMenu->setDelayed(false);
-    actionCollection()->addAction(QL1S("closed_tab_menu"), closedTabsMenu);
-
-    QSignalMapper *tabSignalMapper = new QSignalMapper(this);
     // shortcuts for quickly switching to a tab
+    QSignalMapper *tabSignalMapper = new QSignalMapper(this);
     for (int i = 1; i <= 9; i++)
     {
         a = new KAction(i18n("Switch to Tab %1", i), this);
@@ -452,7 +471,7 @@ void MainWindow::setupActions()
         connect(a, SIGNAL(triggered(bool)), tabSignalMapper, SLOT(map()));
         tabSignalMapper->setMapping(a, i);
     }
-    connect(tabSignalMapper, SIGNAL(mapped(const int)), m_view, SLOT(switchToTab(const int)));
+    connect(tabSignalMapper, SIGNAL(mapped(int)), m_view, SLOT(switchToTab(int)));
 
     // shortcuts for loading favorite pages
     QSignalMapper *favoritesSignalMapper = new QSignalMapper(this);
@@ -464,7 +483,7 @@ void MainWindow::setupActions()
         connect(a, SIGNAL(triggered(bool)), favoritesSignalMapper, SLOT(map()));
         favoritesSignalMapper->setMapping(a, i);
     }
-    connect(favoritesSignalMapper, SIGNAL(mapped(const int)), m_view, SLOT(loadFavorite(const int)));
+    connect(favoritesSignalMapper, SIGNAL(mapped(int)), m_view, SLOT(loadFavorite(int)));
 
     // ============================== Indexed Tab Actions ====================================
     a = new KAction(KIcon("tab-close"), i18n("&Close Tab"), this);
@@ -489,7 +508,7 @@ void MainWindow::setupActions()
     connect(a, SIGNAL(triggered(bool)), m_view->tabBar(), SLOT(detachTab()));
 
     // Bookmark Menu
-    KActionMenu *bmMenu = rApp->bookmarkProvider()->bookmarkActionMenu(this);
+    KActionMenu *bmMenu = rApp->bookmarkManager()->bookmarkActionMenu(this);
     bmMenu->setIcon(KIcon("bookmarks"));
     bmMenu->setDelayed(false);
     bmMenu->setShortcutConfigurable(true);
@@ -500,16 +519,14 @@ void MainWindow::setupActions()
     a = new KAction(KIcon("bookmarks-bar"), i18n("Bookmarks Toolbar"), this);
     a->setCheckable(true);
     actionCollection()->addAction(QL1S("show_bookmarks_toolbar"), a);
+    connect(a, SIGNAL(toggled(bool)), this, SLOT(toggleBookmarkBarVisible(bool)));
 
     // User Agent
     a = new KAction(KIcon("preferences-web-browser-identification"), i18n("Browser Identification"), this);
     actionCollection()->addAction(QL1S("useragent"), a);
-    a->setMenu(m_userAgentMenu);
-    connect(m_userAgentMenu, SIGNAL(aboutToShow()), this, SLOT(populateUserAgentMenu()));
-
-    a = new KAction(KIcon("preferences-web-browser-identification"), i18n("Browser Identification"), this);
-    actionCollection()->addAction(QL1S("UserAgentSettings"), a);
-    connect(a, SIGNAL(triggered(bool)), this, SLOT(showUserAgentSettings()));
+    KMenu *uaMenu = new KMenu(this);
+    a->setMenu(uaMenu);
+    connect(uaMenu, SIGNAL(aboutToShow()), this, SLOT(populateUserAgentMenu()));
 
     // Editable Page
     a = new KAction(KIcon("document-edit"), i18n("Set Editable"), this);
@@ -521,6 +538,16 @@ void MainWindow::setupActions()
     a = new KAction(KIcon("preferences-web-browser-adblock"), i18n("Ad Block"), this);
     actionCollection()->addAction(QL1S("adblock"), a);
     connect(a, SIGNAL(triggered(bool)), rApp->adblockManager(), SLOT(showSettings()));
+
+    // Web Applications
+    a = new KAction(KIcon("applications-internet"), i18n("Create application shortcut"), this);
+    actionCollection()->addAction(QL1S("webapp_shortcut"), a);
+    connect(a, SIGNAL(triggered(bool)), rApp, SLOT(createWebAppShortcut()));
+
+    // Sync action
+    a = new KAction(KIcon("tools-wizard"), i18n("Sync"), this); // FIXME sync icon!!
+    actionCollection()->addAction(QL1S("sync"), a);
+    connect(a, SIGNAL(triggered(bool)), rApp->syncManager(), SLOT(showSettings()));
 }
 
 
@@ -530,7 +557,7 @@ void MainWindow::setupTools()
     toolsAction->setDelayed(false);
     toolsAction->setShortcutConfigurable(true);
     toolsAction->setShortcut(KShortcut(Qt::ALT + Qt::Key_T));
-    m_rekonqMenu = new KMenu(this);
+    m_rekonqMenu = new RekonqMenu(this);
     toolsAction->setMenu(m_rekonqMenu); // dummy menu to have the dropdown arrow
 
     // adding rekonq_tools to rekonq actionCollection
@@ -544,37 +571,43 @@ void MainWindow::setupPanels()
 
     // STEP 1
     // Setup history panel
-    m_historyPanel = new HistoryPanel(i18n("History Panel"), this);
-    connect(m_historyPanel, SIGNAL(openUrl(const KUrl&, const Rekonq::OpenType &)),
-            rApp, SLOT(loadUrl(const KUrl&, const Rekonq::OpenType &)));
+    const QString historyTitle = i18n("History Panel");
+    m_historyPanel = new HistoryPanel(historyTitle, this);
+    connect(m_historyPanel, SIGNAL(openUrl(KUrl,Rekonq::OpenType)),
+            rApp, SLOT(loadUrl(KUrl,Rekonq::OpenType)));
     connect(m_historyPanel, SIGNAL(itemHovered(QString)), this, SLOT(notifyMessage(QString)));
     connect(m_historyPanel, SIGNAL(destroyed()), rApp, SLOT(saveConfiguration()));
 
     addDockWidget(Qt::LeftDockWidgetArea, m_historyPanel);
 
     // setup history panel action
-    a = (KAction *) m_historyPanel->toggleViewAction();
+    a = new KAction(KIcon("view-history"), historyTitle, this);
     a->setShortcut(KShortcut(Qt::CTRL + Qt::Key_H));
-    a->setIcon(KIcon("view-history"));
     actionCollection()->addAction(QL1S("show_history_panel"), a);
+    a->setCheckable(true);
+    connect(a, SIGNAL(triggered(bool)), m_historyPanel, SLOT(setVisible(bool)));
+    connect(m_historyPanel, SIGNAL(visibilityChanged(bool)), a, SLOT(setChecked(bool)));
 
     // STEP 2
     // Setup bookmarks panel
-    m_bookmarksPanel = new BookmarksPanel(i18n("Bookmarks Panel"), this);
-    connect(m_bookmarksPanel, SIGNAL(openUrl(const KUrl&, const Rekonq::OpenType &)),
-            rApp, SLOT(loadUrl(const KUrl&, const Rekonq::OpenType &)));
+    const QString bookmarksTitle = i18n("Bookmarks Panel");
+    m_bookmarksPanel = new BookmarksPanel(bookmarksTitle, this);
+    connect(m_bookmarksPanel, SIGNAL(openUrl(KUrl,Rekonq::OpenType)),
+            rApp, SLOT(loadUrl(KUrl,Rekonq::OpenType)));
     connect(m_bookmarksPanel, SIGNAL(itemHovered(QString)), this, SLOT(notifyMessage(QString)));
     connect(m_bookmarksPanel, SIGNAL(destroyed()), rApp, SLOT(saveConfiguration()));
 
     addDockWidget(Qt::LeftDockWidgetArea, m_bookmarksPanel);
 
-    rApp->bookmarkProvider()->registerBookmarkPanel(m_bookmarksPanel);
+    rApp->bookmarkManager()->registerBookmarkPanel(m_bookmarksPanel);
 
     // setup bookmarks panel action
-    a = (KAction *) m_bookmarksPanel->toggleViewAction();
+    a = new KAction(KIcon("bookmarks-organize"), bookmarksTitle, this);
     a->setShortcut(KShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_B));
-    a->setIcon(KIcon("bookmarks-organize"));
     actionCollection()->addAction(QL1S("show_bookmarks_panel"), a);
+    a->setCheckable(true);
+    connect(a, SIGNAL(triggered(bool)), m_bookmarksPanel, SLOT(setVisible(bool)));
+    connect(m_bookmarksPanel, SIGNAL(visibilityChanged(bool)), a, SLOT(setChecked(bool)));
 
     // STEP 3
     // Setup webinspector panel
@@ -607,15 +640,24 @@ void MainWindow::setupPanels()
 void MainWindow::finalizeGUI(KXMLGUIClient* client)
 {
     KXmlGuiWindow::finalizeGUI(client);
-    //update rekonqMenu when GUI has changed
+
+    // update rekonqMenu when GUI has changed
     KMenu *m = qobject_cast<KMenu*>(factory()->container("rekonqMenu", this));
     if (m)
         m_rekonqMenu->addActions(m->actions());
     else
-        kDebug() << "Could not get the rekonqMenu menu. Maybe the rekonqui.rc file wasn't found. Was"
-                 << "rekonq installed correctly?";
+        kDebug() << " ====================== "
+                 << "Could not get the rekonqMenu menu. Maybe the rekonqui.rc file wasn't found."
+                 << "Was rekonq installed correctly?"
+                 << " ====================== ";
 }
 
+void MainWindow::readProperties(const KConfigGroup& config)
+{
+    Q_UNUSED(config)
+
+    Application::instance()->sessionManager()->restoreMainWindow(this);
+}
 
 void MainWindow::openLocation()
 {
@@ -694,14 +736,14 @@ void MainWindow::preferences()
     QPointer<SettingsDialog> s = new SettingsDialog(this);
 
     // keep us informed when the user changes settings
-    connect(s, SIGNAL(settingsChanged(const QString&)), rApp, SLOT(updateConfiguration()));
+    connect(s, SIGNAL(settingsChanged(QString)), rApp, SLOT(updateConfiguration()));
 
     s->exec();
     delete s;
 }
 
 
-void MainWindow::updateActions()
+void MainWindow::updateHistoryActions()
 {
     bool rekonqPage = currentTab()->page()->isOnRekonqPage();
 
@@ -794,11 +836,16 @@ void MainWindow::printRequested(QWebFrame *frame)
     }
 
     QPrinter printer;
-    QPrintPreviewDialog previewdlg(&printer, this);
+    printer.setDocName(printFrame->title());
+    QPrintDialog *printDialog = KdePrint::createPrintDialog(&printer, this);
 
-    connect(&previewdlg, SIGNAL(paintRequested(QPrinter *)), printFrame, SLOT(print(QPrinter *)));
+    if (printDialog) //check if the Dialog was created
+    {
+        if (printDialog->exec())
+            printFrame->print(&printer);
 
-    previewdlg.exec();
+        delete printDialog;
+    }
 }
 
 
@@ -897,6 +944,7 @@ void MainWindow::updateHighlight()
     }
 }
 
+
 void MainWindow::findSelectedText()
 {
     // FindBar::setVisible() gets the selected text by itself
@@ -911,6 +959,12 @@ void MainWindow::viewFullScreen(bool makeFullScreen)
 }
 
 
+void MainWindow::openDownloadsPage()
+{
+    rApp->loadUrl(KUrl("about:downloads"), Rekonq::NewFocusedTab);
+}
+
+
 void MainWindow::setWidgetsVisible(bool makeVisible)
 {
     // state flags
@@ -919,7 +973,7 @@ void MainWindow::setWidgetsVisible(bool makeVisible)
     static bool bookmarksPanelFlag;
 
     KToolBar *mainBar = toolBar("mainToolBar");
-    KToolBar *bookBar = toolBar("bookmarksToolBar");
+    KToolBar *bookBar = toolBar("bookmarkToolBar");
 
     if (!makeVisible)
     {
@@ -993,23 +1047,20 @@ void MainWindow::viewPageSource()
         srcTab->setPart(pa, tmpUrl);
         srcTab->urlBar()->setQUrl(url.pathOrUrl());
         m_view->setTabText(m_view->currentIndex(), i18n("Source of: ") + url.prettyUrl());
-        updateActions();
+        updateHistoryActions();
     }
     else
         KRun::runUrl(tmpUrl, QL1S("text/plain"), this, false);
 }
 
 
-void MainWindow::homePage(Qt::MouseButtons mouseButtons, Qt::KeyboardModifiers keyboardModifiers)
+void MainWindow::homePage()
 {
     KUrl homeUrl = ReKonfig::useNewTabPage()
                    ? KUrl(QL1S("about:home"))
                    : KUrl(ReKonfig::homePage());
 
-    if (mouseButtons == Qt::MidButton || keyboardModifiers == Qt::ControlModifier)
-        rApp->loadUrl(homeUrl, Rekonq::NewTab);
-    else
-        currentTab()->view()->load(homeUrl);
+    currentTab()->view()->load(homeUrl);
 }
 
 
@@ -1019,29 +1070,42 @@ WebTab *MainWindow::currentTab() const
 }
 
 
-void MainWindow::browserLoading(bool v)
+void MainWindow::updateTabActions()
 {
+    m_loadStopReloadAction->disconnect();
+
+    if (m_view->currentUrlBar()->hasFocus())
+    {
+        m_loadStopReloadAction->disconnect();
+
+        m_loadStopReloadAction->setIcon(KIcon("go-jump-locationbar"));
+        m_loadStopReloadAction->setToolTip(i18n("Go"));
+        m_loadStopReloadAction->setText(i18n("Go"));
+
+        connect(m_loadStopReloadAction, SIGNAL(triggered(bool)), m_view->currentUrlBar(), SLOT(loadDigitedUrl()));
+        return;
+    }
+
     QAction *stop = actionCollection()->action(QL1S("stop"));
     QAction *reload = actionCollection()->action(QL1S("view_redisplay"));
-    if (v)
+
+    if (currentTab()->isPageLoading())
     {
-        disconnect(m_stopReloadAction, SIGNAL(triggered(bool)), reload , SIGNAL(triggered(bool)));
-        m_stopReloadAction->setIcon(KIcon("process-stop"));
-        m_stopReloadAction->setToolTip(i18n("Stop loading the current page"));
-        m_stopReloadAction->setText(i18n("Stop"));
-        connect(m_stopReloadAction, SIGNAL(triggered(bool)), stop, SIGNAL(triggered(bool)));
+        m_loadStopReloadAction->setIcon(KIcon("process-stop"));
+        m_loadStopReloadAction->setToolTip(i18n("Stop loading the current page"));
+        m_loadStopReloadAction->setText(i18n("Stop"));
+        connect(m_loadStopReloadAction, SIGNAL(triggered(bool)), stop, SIGNAL(triggered(bool)));
         stop->setEnabled(true);
     }
     else
     {
-        disconnect(m_stopReloadAction, SIGNAL(triggered(bool)), stop , SIGNAL(triggered(bool)));
-        m_stopReloadAction->setIcon(KIcon("view-refresh"));
-        m_stopReloadAction->setToolTip(i18n("Reload the current page"));
-        m_stopReloadAction->setText(i18n("Reload"));
-        connect(m_stopReloadAction, SIGNAL(triggered(bool)), reload, SIGNAL(triggered(bool)));
+        m_loadStopReloadAction->setIcon(KIcon("view-refresh"));
+        m_loadStopReloadAction->setToolTip(i18n("Reload the current page"));
+        m_loadStopReloadAction->setText(i18n("Reload"));
+        connect(m_loadStopReloadAction, SIGNAL(triggered(bool)), reload, SIGNAL(triggered(bool)));
         stop->setEnabled(false);
 
-        updateActions();
+        updateHistoryActions();
     }
 }
 
@@ -1075,7 +1139,7 @@ void MainWindow::openPrevious(Qt::MouseButtons mouseButtons, Qt::KeyboardModifie
         history->goToItem(*item);
     }
 
-    updateActions();
+    updateHistoryActions();
 }
 
 
@@ -1108,7 +1172,7 @@ void MainWindow::openNext(Qt::MouseButtons mouseButtons, Qt::KeyboardModifiers k
         history->goToItem(*item);
     }
 
-    updateActions();
+    updateHistoryActions();
 }
 
 
@@ -1118,6 +1182,7 @@ void MainWindow::keyPressEvent(QKeyEvent *event)
     if ((event->modifiers() == Qt::ControlModifier) && (event->key() == Qt::Key_Tab))
     {
         emit ctrlTabPressed();
+        event->accept();
         return;
     }
 
@@ -1125,6 +1190,7 @@ void MainWindow::keyPressEvent(QKeyEvent *event)
     if ((event->modifiers() == Qt::ControlModifier + Qt::ShiftModifier) && (event->key() == Qt::Key_Backtab))
     {
         emit shiftCtrlTabPressed();
+        event->accept();
         return;
     }
 
@@ -1159,6 +1225,7 @@ bool MainWindow::event(QEvent *event)
             }
         }
     }
+
     return KMainWindow::event(event);
 }
 
@@ -1183,6 +1250,8 @@ void MainWindow::notifyMessage(const QString &msg, Rekonq::Notify status)
     case Rekonq::Success:
     case Rekonq::Error:
     case Rekonq::Download:
+        m_hidePopupTimer->start(3000);
+        break;
     default:
         break;
     }
@@ -1209,6 +1278,11 @@ void MainWindow::notifyMessage(const QString &msg, Rekonq::Notify status)
     m_popup->setFixedSize(labelSize);
     m_popup->setText(fm.elidedText(msgToShow, Qt::ElideMiddle, labelSize.width() - 2 * margin));
 
+    // NOTE: while currentFrame should NEVER be null
+    // we are checking here its existence cause of bug:264187
+    if (!tab->page()->currentFrame())
+        return;
+
     const bool horizontalScrollbarIsVisible = tab->page()->currentFrame()->scrollBarMaximum(Qt::Horizontal);
     const bool verticalScrollbarIsVisible = tab->page()->currentFrame()->scrollBarMaximum(Qt::Vertical);
     const bool actionBarsVisible = m_findBar->isVisible() || m_zoomBar->isVisible();
@@ -1295,7 +1369,7 @@ void MainWindow::clearPrivateData()
             path.remove("rekonq");
             QDir cacheDir(path);
             QStringList fileList = cacheDir.entryList();
-            foreach(const QString & str, fileList)
+            Q_FOREACH(const QString & str, fileList)
             {
                 QFile file(path + str);
                 file.remove();
@@ -1433,65 +1507,6 @@ void MainWindow::openActionTab(QAction* action)
 }
 
 
-void MainWindow::setUserAgent()
-{
-    QAction *sender = static_cast<QAction *>(QObject::sender());
-
-    QString info;
-    QString desc = sender->text();
-    int uaIndex = sender->data().toInt();
-
-    KUrl url = currentTab()->url();
-    UserAgentInfo uaInfo;
-    uaInfo.setUserAgentForHost(uaIndex, url.host());
-    currentTab()->page()->triggerAction(QWebPage::Reload);
-}
-
-
-void MainWindow::populateUserAgentMenu()
-{
-    bool defaultUA = true;
-    KUrl url = currentTab()->url();
-
-    QAction *a, *defaultAction;
-
-    m_userAgentMenu->clear();
-
-    defaultAction = new QAction(i18nc("Default rekonq user agent", "Default"), this);
-    defaultAction->setData(-1);
-    defaultAction->setCheckable(true);
-    connect(defaultAction, SIGNAL(triggered(bool)), this, SLOT(setUserAgent()));
-
-    m_userAgentMenu->addAction(defaultAction);
-    m_userAgentMenu->addSeparator();
-
-    UserAgentInfo uaInfo;
-    QStringList UAlist = uaInfo.availableUserAgents();
-    int uaIndex = uaInfo.uaIndexForHost(currentTab()->url().host());
-
-    for (int i = 0; i < UAlist.count(); ++i)
-    {
-        QString uaDesc = UAlist.at(i);
-
-        a = new QAction(uaDesc, this);
-        a->setData(i);
-        a->setCheckable(true);
-        connect(a, SIGNAL(triggered(bool)), this, SLOT(setUserAgent()));
-
-        if (i == uaIndex)
-        {
-            a->setChecked(true);
-            defaultUA = false;
-        }
-        m_userAgentMenu->addAction(a);
-    }
-    defaultAction->setChecked(defaultUA);
-
-    m_userAgentMenu->addSeparator();
-    m_userAgentMenu->addAction(actionByName("UserAgentSettings"));
-}
-
-
 void MainWindow::enableNetworkAnalysis(bool b)
 {
     currentTab()->page()->enableNetworkAnalyzer(b);
@@ -1520,24 +1535,13 @@ void MainWindow::setupBookmarksAndToolsShortcuts()
     if (toolsButton)
     {
         connect(actionByName(QL1S("rekonq_tools")), SIGNAL(triggered()), toolsButton, SLOT(showMenu()));
+
+        // HACK: set button widget in rekonq menu
+        m_rekonqMenu->setButtonWidget(toolsButton);
     }
 }
 
 
-void MainWindow::showUserAgentSettings()
-{
-    QPointer<KDialog> dialog = new KDialog(this);
-    dialog->setCaption(i18nc("@title:window", "User Agent Settings"));
-    dialog->setButtons(KDialog::Ok);
-
-    UserAgentWidget widget;
-    dialog->setMainWidget(&widget);
-    dialog->exec();
-
-    dialog->deleteLater();
-}
-
-
 void MainWindow::moveEvent(QMoveEvent *event)
 {
     if (m_hidePopupTimer)
@@ -1572,3 +1576,10 @@ void MainWindow::closeEvent(QCloseEvent *event)
     kDebug() << "CLOSING WINDOW...";
     KXmlGuiWindow::closeEvent(event);
 }
+
+
+void MainWindow::populateUserAgentMenu()
+{
+    KMenu *uaMenu = static_cast<KMenu *>(QObject::sender());
+    rApp->userAgentManager()->populateUAMenuForTabUrl(uaMenu, currentTab());
+}
