@@ -2,7 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2009-2011 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2009-2012 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2010 by Matthieu Gicquel <matgic78 at gmail dot com>
 *
 *
@@ -57,12 +57,6 @@ public:
      */
     void generate(const KUrl &url = KUrl("about:home"));
 
-    /**
-     * This method updates thumbs, removing loading previews
-     * and providing a real picture
-     */
-    void snapFinished();
-
 private:
     // these are the "high-level" functions to build the new tab page.
     // Basically, you call browsingMenu + one of the *Page methods
@@ -74,6 +68,7 @@ private:
     void bookmarksPage();
     void closedTabsPage();
     void downloadsPage();
+    void tabsPage();
 
     // --------------------------------------------------------------------------
     // "low-level" functions
@@ -81,8 +76,9 @@ private:
 
     // Previews handling
     QWebElement emptyPreview(int index);
-    QWebElement loadingPreview(int index, const KUrl &url);
     QWebElement validPreview(int index, const KUrl &url, const QString &title);
+    QWebElement tabPreview(int winIndex, int tabIndex, const KUrl &url, const QString &title);
+    QWebElement closedTabPreview(int index, const KUrl &url, const QString &title);
 
     void removePreview(int index);
 
@@ -94,6 +90,7 @@ private:
     void hideControls(QWebElement e);
     void showControls(QWebElement e);
     void setupPreview(QWebElement e, int index);
+    void setupTabPreview(QWebElement e, int winIndex, int tabIndex);
 
     void createBookItem(const KBookmark &bookmark, QWebElement parent);
 
