@@ -3,7 +3,7 @@
 * This file is a part of the rekonq project
 *
 * Copyright (C) 2008 Benjamin C. Meyer <ben@meyerhome.net>
-* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2012 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2009 by Paweł Prażak <pawelprazak at gmail dot com>
 * Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
 *
@@ -43,6 +43,7 @@
 #include "websnap.h"
 #include "tabhighlighteffect.h"
 #include "tabpreviewpopup.h"
+#include "searchengine.h"
 
 // KDE Includes
 #include <KActionMenu>
@@ -81,13 +82,14 @@ TabBar::TabBar(QWidget *parent)
 
     setContextMenuPolicy(Qt::CustomContextMenu);
 
-    connect(this, SIGNAL(contextMenu(int, const QPoint &)), this, SLOT(contextMenu(int, const QPoint &)));
-    connect(this, SIGNAL(emptyAreaContextMenu(const QPoint &)), this, SLOT(emptyAreaContextMenu(const QPoint &)));
+    connect(this, SIGNAL(contextMenu(int,QPoint)), this, SLOT(contextMenu(int,QPoint)));
+    connect(this, SIGNAL(emptyAreaContextMenu(QPoint)), this, SLOT(emptyAreaContextMenu(QPoint)));
 
     connect(m_animationMapper, SIGNAL(mapped(int)), this, SLOT(removeAnimation(int)));
     setGraphicsEffect(m_tabHighlightEffect);
 
     setAnimatedTabHighlighting(ReKonfig::animatedTabHighlighting());
+    setAcceptDrops(true);
 }
 
 
@@ -302,7 +304,7 @@ void TabBar::mousePressEvent(QMouseEvent *event)
 
 void TabBar::contextMenu(int tab, const QPoint &pos)
 {
-    setupHistoryActions();
+    KActionMenu *closedTabsMenu = setupHistoryActions();
 
     m_actualIndex = tab;
 
@@ -312,30 +314,37 @@ void TabBar::contextMenu(int tab, const QPoint &pos)
     menu.addAction(mainWindow->actionByName(QL1S("new_tab")));
     menu.addAction(mainWindow->actionByName(QL1S("clone_tab")));
     if (count() > 1)
+    {
         menu.addAction(mainWindow->actionByName(QL1S("detach_tab")));
+    }
     menu.addAction(mainWindow->actionByName(QL1S("open_last_closed_tab")));
-    menu.addAction(mainWindow->actionByName(QL1S("closed_tab_menu")));
+    menu.addAction(closedTabsMenu);
     menu.addSeparator();
     menu.addAction(mainWindow->actionByName(QL1S("close_tab")));
-    menu.addAction(mainWindow->actionByName(QL1S("close_other_tabs")));
+    if (count() > 1)
+    {
+        menu.addAction(mainWindow->actionByName(QL1S("close_other_tabs")));
+    }
     menu.addSeparator();
     menu.addAction(mainWindow->actionByName(QL1S("reload_tab")));
-    menu.addAction(mainWindow->actionByName(QL1S("reload_all_tabs")));
-
+    if (count() > 1)
+    {
+        menu.addAction(mainWindow->actionByName(QL1S("reload_all_tabs")));
+    }
     menu.exec(pos);
 }
 
 
 void TabBar::emptyAreaContextMenu(const QPoint &pos)
 {
-    setupHistoryActions();
+    KActionMenu *closedTabsMenu = setupHistoryActions();
 
     KMenu menu;
     MainWindow *mainWindow = rApp->mainWindow();
 
     menu.addAction(mainWindow->actionByName(QL1S("new_tab")));
     menu.addAction(mainWindow->actionByName(QL1S("open_last_closed_tab")));
-    menu.addAction(mainWindow->actionByName(QL1S("closed_tab_menu")));
+    menu.addAction(closedTabsMenu);
     menu.addSeparator();
     menu.addAction(mainWindow->actionByName(QL1S("reload_all_tabs")));
 
@@ -365,35 +374,36 @@ void TabBar::tabRemoved(int index)
 }
 
 
-void TabBar::setupHistoryActions()
+KActionMenu *TabBar::setupHistoryActions()
 {
     MainWindow *w = rApp->mainWindow();
     MainView *mv = qobject_cast<MainView *>(parent());
 
     QAction *openLastClosedTabAction = w->actionByName(QL1S("open_last_closed_tab"));
-    openLastClosedTabAction->setEnabled(mv->recentlyClosedTabs().size() > 0);
 
-    // update closed tabs menu
-    KActionMenu *am = qobject_cast<KActionMenu *>(w->actionByName(QL1S("closed_tab_menu")));
-    if (!am)
-        return;
+    bool closedTabsAvailable = (mv->recentlyClosedTabs().size() > 0);
+    openLastClosedTabAction->setEnabled(closedTabsAvailable);
 
-    bool isEnabled = (mv->recentlyClosedTabs().size() > 0);
-    am->setEnabled(isEnabled);
+    KActionMenu *am = new KActionMenu(KIcon("tab-new"), i18n("Closed Tabs"), this);
+    am->setDelayed(false);
+    am->setEnabled(closedTabsAvailable);
 
     if (am->menu())
         am->menu()->clear();
 
-    if (!isEnabled)
-        return;
+    if (!closedTabsAvailable)
+        return am;
 
-    Q_FOREACH(const TabHistory & item, mv->recentlyClosedTabs())
+    for (int i = 0; i < mv->recentlyClosedTabs().count(); ++i)
     {
+        TabHistory item = mv->recentlyClosedTabs().at(i);
         KAction *a = new KAction(rApp->iconManager()->iconForUrl(item.url), item.title, this);
-        a->setData(item.history);
+        a->setData(i);
         connect(a, SIGNAL(triggered()), mv, SLOT(openClosedTab()));
         am->addAction(a);
     }
+
+    return am;
 }
 
 
@@ -479,3 +489,54 @@ void TabBar::setAnimatedTabHighlighting(bool enabled)
         }
     }
 }
+
+
+void TabBar::dropEvent(QDropEvent* event)
+{
+    if (event->mimeData()->hasUrls())
+    {
+        int urlCount = event->mimeData()->urls().count();
+        if (urlCount > 1)
+        {
+            Q_FOREACH(const QUrl & url, event->mimeData()->urls())
+            rApp->loadUrl(url, Rekonq::NewTab);
+        }
+        else
+            rApp->loadUrl(event->mimeData()->urls().first(), Rekonq::NewFocusedTab);
+    }
+    else if (event->mimeData()->hasText())
+    {
+        //In case the text is a valid URL
+        if (isURLValid(event->mimeData()->text()))
+            rApp->loadUrl(KUrl(event->mimeData()->text()), Rekonq::NewFocusedTab);
+        else
+        {
+            KService::Ptr defaultSearchEngine = SearchEngine::defaultEngine();
+            if (defaultSearchEngine)
+                rApp->loadUrl(KUrl(SearchEngine::buildQuery(defaultSearchEngine, event->mimeData()->text())), Rekonq::NewFocusedTab);
+        }
+    }
+    KTabBar::dropEvent(event);
+}
+
+
+void TabBar::dragEnterEvent(QDragEnterEvent* event)
+{
+    if (event->mimeData()->hasUrls() || event->mimeData()->hasText())
+        event->acceptProposedAction();
+    else
+        KTabBar::dragEnterEvent(event);
+}
+
+
+bool TabBar::isURLValid(const QString &url)
+{
+    QString editedURL = url;
+    bool isValid = false;
+    if (editedURL.startsWith(QL1S("http://")) || editedURL.startsWith(QL1S("https://")) || editedURL.startsWith(QL1S("ftp://")))
+        editedURL = editedURL.remove(QRegExp("(http|https|ftp)://"));
+    if (editedURL.contains(QL1C('.')) && editedURL.indexOf(QL1C('.')) > 0 && editedURL.indexOf(QL1C('.')) < editedURL.length() && !editedURL.trimmed().contains(QL1C(' '))
+            && QUrl::fromUserInput(editedURL).isValid())
+        isValid = true;
+    return isValid;
+}
