@@ -2,7 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2012 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2009 by Domrachev Alexandr <alexandr.domrachev@gmail.com>
 * Copyright (C) 2009 by Paweł Prażak <pawelprazak at gmail dot com>
 * Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
@@ -41,7 +41,7 @@
 #include "webpage.h"
 #include "webview.h"
 #include "completionwidget.h"
-#include "bookmarkprovider.h"
+#include "bookmarkmanager.h"
 #include "bookmarkowner.h"
 #include "bookmarkwidget.h"
 #include "iconmanager.h"
@@ -81,6 +81,21 @@ void IconButton::mouseReleaseEvent(QMouseEvent* event)
 // -----------------------------------------------------------------------------------------------------------
 
 
+QString guessUrlWithCustomFirstLevel(const QString &str1, const QString &str2)
+{
+    QUrl url(QL1S("http://www.") + str1);
+    QString host = url.host().toLower();
+    if (!host.endsWith(str2, Qt::CaseInsensitive))
+    {
+        host += str2;
+        url.setHost(host);
+    }
+    return url.toString();
+}
+
+// -----------------------------------------------------------------------------------------------------------
+
+
 UrlBar::UrlBar(QWidget *parent)
     : KLineEdit(parent)
     , _tab(0)
@@ -88,7 +103,7 @@ UrlBar::UrlBar(QWidget *parent)
     , _suggestionTimer(new QTimer(this))
 {
     // initial style
-    setStyleSheet(QString("UrlBar { padding: 0 0 0 %1px;} ").arg(_icon->sizeHint().width()));
+    setStyleSheet(QString("UrlBar { padding: 2px 0 2px %1px;} ").arg(_icon->sizeHint().width()));
 
     // doesn't show the clear button
     setClearButtonShown(false);
@@ -112,15 +127,18 @@ UrlBar::UrlBar(QWidget *parent)
 
     connect(_tab, SIGNAL(loadProgressing()), this, SLOT(update()));
 
-    connect(_tab->view(), SIGNAL(urlChanged(const QUrl &)), this, SLOT(setQUrl(const QUrl &)));
+    connect(_tab->view(), SIGNAL(urlChanged(QUrl)), this, SLOT(setQUrl(QUrl)));
     connect(_tab->view(), SIGNAL(loadFinished(bool)), this, SLOT(loadFinished()));
     connect(_tab->view(), SIGNAL(loadStarted()), this, SLOT(clearRightIcons()));
     connect(_tab->view(), SIGNAL(iconChanged()), this, SLOT(refreshFavicon()));
 
     // search icon
-    connect(rApp->opensearchManager(), SIGNAL(openSearchEngineAdded(const QString &, const QString &, const QString &)),
+    connect(rApp->opensearchManager(), SIGNAL(openSearchEngineAdded(QString,QString,QString)),
             this, SLOT(updateRightIcons()));
 
+    // bookmark icon
+    connect(rApp->bookmarkManager(), SIGNAL(bookmarksUpdated()),this, SLOT(updateRightIcons()));
+
     _suggestionTimer->setSingleShot(true);
     connect(_suggestionTimer, SIGNAL(timeout()), this, SLOT(suggest()));
 
@@ -153,7 +171,7 @@ void UrlBar::setQUrl(const QUrl& url)
 }
 
 
-void UrlBar::activated(const KUrl& url, Rekonq::OpenType type)
+void UrlBar::loadRequestedUrl(const KUrl& url, Rekonq::OpenType type)
 {
     activateSuggestions(false);
     clearFocus();
@@ -162,6 +180,21 @@ void UrlBar::activated(const KUrl& url, Rekonq::OpenType type)
 }
 
 
+void UrlBar::loadDigitedUrl()
+{
+    UrlResolver res(text());
+    UrlSearchList list = res.orderedSearchItems();
+    if (list.isEmpty())
+    {
+        loadRequestedUrl(KUrl(text()));
+    }
+    else
+    {
+        loadRequestedUrl(list.first().url);
+    }
+}
+
+
 void UrlBar::paintEvent(QPaintEvent *event)
 {
     KColorScheme colorScheme(palette().currentColorGroup());
@@ -225,7 +258,7 @@ void UrlBar::paintEvent(QPaintEvent *event)
     // you need this before our code to draw inside the line edit..
     KLineEdit::paintEvent(event);
 
-    if (text().isEmpty() && progr == 0)
+    if (text().isEmpty() && (progr == 0 || progr == 100))
     {
         QStyleOptionFrame option;
         initStyleOption(&option);
@@ -242,60 +275,43 @@ void UrlBar::paintEvent(QPaintEvent *event)
 
 void UrlBar::keyPressEvent(QKeyEvent *event)
 {
-    // this handles the Modifiers + Return key combinations
     QString currentText = text().trimmed();
-    if (event->modifiers() == Qt::AltModifier)
-    {
-        if (event->key() == Qt::Key_Return || event->key() == Qt::Key_Enter)
-        {
-            activated(currentText, Rekonq::NewFocusedTab);
-        }
-    }
-    if ((event->key() == Qt::Key_Enter || event->key() == Qt::Key_Return)
-            && !currentText.startsWith(QL1S("http://"), Qt::CaseInsensitive)
-            && event->modifiers() != Qt::NoModifier)
-    {
-        QString append;
-        if (event->modifiers() == Qt::ControlModifier)
-        {
-            append = QL1S(".com");
-        }
-        else if (event->modifiers() == (Qt::ControlModifier | Qt::ShiftModifier))
-        {
-            append = QL1S(".org");
-        }
-        else if (event->modifiers() == Qt::ShiftModifier)
-        {
-            append = QL1S(".net");
-        }
 
-        if (!append.isEmpty())
-        {
-            QUrl url(QL1S("http://www.") + currentText);
-            QString host = url.host();
-            if (!host.endsWith(append, Qt::CaseInsensitive))
-            {
-                host += append;
-                url.setHost(host);
-            }
+    if (currentText.isEmpty())
+        return KLineEdit::keyPressEvent(event);
 
-            // now, load it!
-            activated(url);
+    // this handles the Modifiers + Return key combinations
+    if (event->key() == Qt::Key_Return || event->key() == Qt::Key_Enter)
+    {
+        switch (event->modifiers())
+        {
+        case Qt::AltModifier:
+            loadRequestedUrl(currentText, Rekonq::NewFocusedTab);
+            break;
+
+        case Qt::ControlModifier:
+            loadRequestedUrl(guessUrlWithCustomFirstLevel(currentText, QL1S(".com")));
+            break;
+
+        case 0x06000000: // Qt::ControlModifier | Qt::ShiftModifier:
+            loadRequestedUrl(guessUrlWithCustomFirstLevel(currentText, QL1S(".org")));
+            break;
+
+        case Qt::ShiftModifier:
+            loadRequestedUrl(guessUrlWithCustomFirstLevel(currentText, QL1S(".net")));
+            break;
+
+        default:
+            loadRequestedUrl(currentText);
+            break;
         }
     }
 
-    else if ((event->key() == Qt::Key_Enter || event->key() == Qt::Key_Return)
-             && !currentText.isEmpty())
-    {
-        loadTyped(currentText);
-    }
-
-    else if (event->key() == Qt::Key_Escape)
+    if (event->key() == Qt::Key_Escape)
     {
         clearFocus();
-        if (text() != rApp->mainWindow()->currentTab()->view()->url().toString()
-                && !rApp->mainWindow()->currentTab()->view()->url().toString().startsWith(QL1S("about")))
-            setText(rApp->mainWindow()->currentTab()->view()->url().toString());
+        if (!(_tab->url().protocol() == QL1S("about")))
+            setText(_tab->url().url());
         event->accept();
     }
 
@@ -306,6 +322,7 @@ void UrlBar::keyPressEvent(QKeyEvent *event)
 void UrlBar::focusInEvent(QFocusEvent *event)
 {
     activateSuggestions(true);
+    rApp->mainWindow()->updateTabActions();
 
     KLineEdit::focusInEvent(event);
 }
@@ -321,14 +338,14 @@ void UrlBar::dropEvent(QDropEvent *event)
         if (url.isValid())
         {
             setQUrl(url);
-            activated(text());
+            loadRequestedUrl(text());
             return;
         }
     }
 
     // handles everything else
     KLineEdit::dropEvent(event);
-    activated(text());
+    loadRequestedUrl(text());
 }
 
 
@@ -340,6 +357,9 @@ void UrlBar::loadFinished()
         return;
     }
 
+    // Make sure icons aren't duplicated
+    clearRightIcons();
+
     // show Favorite Icon
     if (ReKonfig::previewUrls().contains(_tab->url().url()))
     {
@@ -384,7 +404,7 @@ void UrlBar::loadFinished()
     // removing this code (where setStyleSheet automatically calls update) needs adding again
     // an update call
     int rightIconWidth = 25 * (_rightIconsList.count());
-    setStyleSheet(QString("UrlBar { padding: 0 %2px 0 %1px;} ").arg(_icon->sizeHint().width()).arg(rightIconWidth));
+    setStyleSheet(QString("UrlBar { padding: 2px %2px 2px %1px;} ").arg(_icon->sizeHint().width()).arg(rightIconWidth));
 }
 
 
@@ -399,12 +419,11 @@ void UrlBar::showBookmarkInfo(QPoint pos)
     if (_tab->url().scheme() == QL1S("about"))
         return;
 
-    KBookmark bookmark = rApp->bookmarkProvider()->bookmarkForUrl(_tab->url());
+    KBookmark bookmark = rApp->bookmarkManager()->bookmarkForUrl(_tab->url());
 
     if (bookmark.isNull())
     {
-        bookmark = rApp->bookmarkProvider()->bookmarkOwner()->bookmarkCurrentPage();
-        updateRightIcons();
+        bookmark = rApp->bookmarkManager()->owner()->bookmarkCurrentPage();
     }
     else
     {
@@ -419,18 +438,11 @@ void UrlBar::updateRightIcons()
 {
     if (!_tab->isPageLoading())
     {
-        clearRightIcons();
         loadFinished();
     }
 }
 
 
-void UrlBar::loadTyped(const QString &text)
-{
-    activated(KUrl(text));
-}
-
-
 void UrlBar::activateSuggestions(bool b)
 {
     if (b)
@@ -439,15 +451,15 @@ void UrlBar::activateSuggestions(bool b)
         {
             _box = new CompletionWidget(this);
             installEventFilter(_box.data());
-            connect(_box.data(), SIGNAL(chosenUrl(const KUrl &, Rekonq::OpenType)), this, SLOT(activated(const KUrl &, Rekonq::OpenType)));
+            connect(_box.data(), SIGNAL(chosenUrl(KUrl,Rekonq::OpenType)), this, SLOT(loadRequestedUrl(KUrl,Rekonq::OpenType)));
 
             // activate suggestions on edit text
-            connect(this, SIGNAL(textChanged(const QString &)), this, SLOT(detectTypedString(const QString &)));
+            connect(this, SIGNAL(textChanged(QString)), this, SLOT(detectTypedString(QString)));
         }
     }
     else
     {
-        disconnect(this, SIGNAL(textChanged(const QString &)), this, SLOT(detectTypedString(const QString &)));
+        disconnect(this, SIGNAL(textChanged(QString)), this, SLOT(detectTypedString(QString)));
         removeEventFilter(_box.data());
         if (!_box.isNull())
             _box.data()->deleteLater();
@@ -557,7 +569,7 @@ IconButton *UrlBar::addRightIcon(UrlBar::icon ic)
         rightIcon->setToolTip(i18n("Show SSL Info"));
         break;
     case UrlBar::BK:
-        if (rApp->bookmarkProvider()->bookmarkForUrl(_tab->url()).isNull())
+        if (rApp->bookmarkManager()->bookmarkForUrl(_tab->url()).isNull())
         {
             rightIcon->setIcon(KIcon("bookmarks").pixmap(32, 32, QIcon::Disabled));
             rightIcon->setToolTip(i18n("Bookmark this page"));
@@ -686,7 +698,7 @@ void UrlBar::bookmarkContextMenu(QPoint pos)
     KMenu menu(this);
     QAction *qa;
 
-    if (!rApp->bookmarkProvider()->bookmarkForUrl(_tab->url()).isNull())
+    if (!rApp->bookmarkManager()->bookmarkForUrl(_tab->url()).isNull())
     {
         qa = new KAction(KIcon("bookmarks"), i18n("Edit Bookmark"), this);
         connect(qa, SIGNAL(triggered(bool)), this, SLOT(showBookmarkDialog()));
@@ -724,7 +736,7 @@ void UrlBar::addFavorite()
 
 void UrlBar::pasteAndGo()
 {
-    activated(rApp->clipboard()->text());
+    loadRequestedUrl(rApp->clipboard()->text());
 }
 
 
@@ -732,7 +744,7 @@ void UrlBar::pasteAndSearch()
 {
     KService::Ptr defaultEngine = SearchEngine::defaultEngine();
     if (defaultEngine)
-        activated(KUrl(SearchEngine::buildQuery(defaultEngine, rApp->clipboard()->text())));
+        loadRequestedUrl(KUrl(SearchEngine::buildQuery(defaultEngine, rApp->clipboard()->text())));
 }
 
 
