@@ -6,7 +6,7 @@
 * Copyright (C) 2008 Dirk Mueller <mueller@kde.org>
 * Copyright (C) 2008 Urs Wolfer <uwolfer @ kde.org>
 * Copyright (C) 2008 Michael Howell <mhowell123@gmail.com>
-* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2012 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2010 by Matthieu Gicquel <matgic78 at gmail dot com>
 * Copyright (C) 2009-2010 Dawit Alemayehu <adawit at kde dot org>
 *
@@ -45,45 +45,33 @@
 #include "mainview.h"
 #include "mainwindow.h"
 #include "networkaccessmanager.h"
-#include "newtabpage.h"
 #include "urlbar.h"
 #include "webpluginfactory.h"
+#include "websnap.h"
 #include "webtab.h"
 #include "searchengine.h"
 #include "sslwidget.h"
 #include "sslinfodialog.h"
 
 // KDE Includes
-#include <KIO/Job>
-#include <KIO/RenameDialog>
-#include <KIO/JobUiDelegate>
-
 #include <KTemporaryFile>
 #include <KStandardDirs>
-#include <KFileDialog>
 #include <KJobUiDelegate>
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <KMimeTypeTrader>
 #include <KService>
-#include <KToolInvocation>
 #include <KWebWallet>
 #include <KProtocolInfo>
 #include <KRun>
 
+#include <KIO/JobUiDelegate>
+
 #include <kparts/browseropenorsavequestion.h>
 
-#include <kio/scheduler.h>
-
 // Qt Includes
-#include <QtCore/QFileInfo>
-
 #include <QtGui/QTextDocument>
 
-#include <QtDBus/QDBusConnection>
-#include <QtDBus/QDBusConnectionInterface>
-#include <QtDBus/QDBusInterface>
-
 
 // Returns true if the scheme and domain of the two urls match...
 static bool domainSchemeMatch(const QUrl& u1, const QUrl& u2)
@@ -107,46 +95,6 @@ static bool domainSchemeMatch(const QUrl& u1, const QUrl& u2)
 }
 
 
-// NOTE
-// These 2 functions have been copied from the KWebPage class to implement a local version of the downloadResponse method.
-// In this way, we can easily provide the extra functionality we need:
-// 1. KGet Integration
-// 2. Save downloads history
-static void extractSuggestedFileName(const QNetworkReply* reply, QString& fileName)
-{
-    fileName.clear();
-    const KIO::MetaData& metaData = reply->attribute(static_cast<QNetworkRequest::Attribute>(KIO::AccessManager::MetaData)).toMap();
-    if (metaData.value(QL1S("content-disposition-type")).compare(QL1S("attachment"), Qt::CaseInsensitive) == 0)
-        fileName = metaData.value(QL1S("content-disposition-filename"));
-
-    if (!fileName.isEmpty())
-        return;
-
-    if (!reply->hasRawHeader("Content-Disposition"))
-        return;
-
-    const QString value(QL1S(reply->rawHeader("Content-Disposition").simplified().constData()));
-    if (value.startsWith(QL1S("attachment"), Qt::CaseInsensitive) || value.startsWith(QL1S("inline"), Qt::CaseInsensitive))
-    {
-        const int length = value.size();
-        int pos = value.indexOf(QL1S("filename"), 0, Qt::CaseInsensitive);
-        if (pos > -1)
-        {
-            pos += 9;
-            while (pos < length && (value.at(pos) == QL1C(' ') || value.at(pos) == QL1C('=') || value.at(pos) == QL1C('"')))
-                pos++;
-
-            int endPos = pos;
-            while (endPos < length && value.at(endPos) != QL1C('"') && value.at(endPos) != QL1C(';'))
-                endPos++;
-
-            if (endPos > pos)
-                fileName = value.mid(pos, (endPos - pos)).trimmed();
-        }
-    }
-}
-
-
 static void extractMimeType(const QNetworkReply* reply, QString& mimeType)
 {
     mimeType.clear();
@@ -169,87 +117,6 @@ static void extractMimeType(const QNetworkReply* reply, QString& mimeType)
 }
 
 
-static bool downloadResource(const KUrl& srcUrl, const KIO::MetaData& metaData = KIO::MetaData(),
-                             QWidget * parent = 0, const QString & suggestedName = QString())
-{
-    KUrl destUrl;
-
-    int result = KIO::R_OVERWRITE;
-    const QString fileName((suggestedName.isEmpty() ? srcUrl.fileName() : suggestedName));
-
-    do
-    {
-        // follow bug:184202 fixes
-        destUrl = KFileDialog::getSaveFileName(KUrl::fromPath(fileName), QString(), parent);
-
-        if (destUrl.isEmpty())
-            return false;
-
-        if (destUrl.isLocalFile())
-        {
-            QFileInfo finfo(destUrl.toLocalFile());
-            if (finfo.exists())
-            {
-                QDateTime now = QDateTime::currentDateTime();
-                QPointer<KIO::RenameDialog> dlg = new KIO::RenameDialog(parent,
-                        i18n("Overwrite File?"),
-                        srcUrl,
-                        destUrl,
-                        KIO::RenameDialog_Mode(KIO::M_OVERWRITE | KIO::M_SKIP),
-                        -1,
-                        finfo.size(),
-                        now.toTime_t(),
-                        finfo.created().toTime_t(),
-                        now.toTime_t(),
-                        finfo.lastModified().toTime_t()
-                                                                       );
-                result = dlg->exec();
-                delete dlg;
-            }
-        }
-    }
-    while (result == KIO::R_CANCEL && destUrl.isValid());
-
-    // Save download history
-    DownloadItem *item = rApp->downloadManager()->addDownload(srcUrl.pathOrUrl(), destUrl.pathOrUrl());
-
-    if (!KStandardDirs::findExe("kget").isNull() && ReKonfig::kgetDownload())
-    {
-        //KGet integration:
-        if (!QDBusConnection::sessionBus().interface()->isServiceRegistered("org.kde.kget"))
-        {
-            KToolInvocation::kdeinitExecWait("kget");
-        }
-        QDBusInterface kget("org.kde.kget", "/KGet", "org.kde.kget.main");
-        if (!kget.isValid())
-            return false;
-
-        QDBusMessage transfer = kget.call(QL1S("addTransfer"), srcUrl.prettyUrl(), destUrl.prettyUrl(), true);
-        if (transfer.arguments().isEmpty())
-            return true;
-
-        const QString transferPath = transfer.arguments().first().toString();
-        item->setKGetTransferDbusPath(transferPath);
-        return true;
-    }
-
-    KIO::Job *job = KIO::file_copy(srcUrl, destUrl, -1, KIO::Overwrite);
-    if (item)
-    {
-        QObject::connect(job, SIGNAL(percent(KJob *, unsigned long)), item, SLOT(updateProgress(KJob *, unsigned long)));
-        QObject::connect(job, SIGNAL(finished(KJob *)), item, SLOT(onFinished(KJob*)));
-    }
-
-    if (!metaData.isEmpty())
-        job->setMetaData(metaData);
-
-    job->addMetaData(QL1S("MaxCacheSize"), QL1S("0")); // Don't store in http cache.
-    job->addMetaData(QL1S("cache"), QL1S("cache")); // Use entry from cache if available.
-    job->uiDelegate()->setAutoErrorHandlingEnabled(true);
-    return true;
-}
-
-
 // ---------------------------------------------------------------------------------
 
 
@@ -260,7 +127,7 @@ WebPage::WebPage(QWidget *parent)
 {
     // ----- handling unsupported content...
     setForwardUnsupportedContent(true);
-    connect(this, SIGNAL(unsupportedContent(QNetworkReply *)), this, SLOT(handleUnsupportedContent(QNetworkReply *)));
+    connect(this, SIGNAL(unsupportedContent(QNetworkReply*)), this, SLOT(handleUnsupportedContent(QNetworkReply*)));
 
     // ----- rekonq Network Manager
     NetworkAccessManager *manager = new NetworkAccessManager(this);
@@ -286,12 +153,12 @@ WebPage::WebPage(QWidget *parent)
     // ----- last stuffs
     connect(manager, SIGNAL(finished(QNetworkReply*)), this, SLOT(manageNetworkErrors(QNetworkReply*)));
 
-    connect(this, SIGNAL(downloadRequested(const QNetworkRequest &)), this, SLOT(downloadRequest(const QNetworkRequest &)));
+    connect(this, SIGNAL(downloadRequested(QNetworkRequest)), this, SLOT(downloadRequest(QNetworkRequest)));
     connect(this, SIGNAL(loadStarted()), this, SLOT(loadStarted()));
     connect(this, SIGNAL(loadFinished(bool)), this, SLOT(loadFinished(bool)));
 
     // protocol handler signals
-    connect(&_protHandler, SIGNAL(downloadUrl(const KUrl &)), this, SLOT(downloadUrl(const KUrl &)));
+    connect(&_protHandler, SIGNAL(downloadUrl(KUrl)), this, SLOT(downloadUrl(KUrl)));
 
     connect(rApp->iconManager(), SIGNAL(iconChanged()), mainFrame(), SIGNAL(iconChanged()));
 }
@@ -300,6 +167,11 @@ WebPage::WebPage(QWidget *parent)
 WebPage::~WebPage()
 {
     disconnect();
+
+    QPixmap preview = WebSnap::renderPagePreview(*this);
+    QString path = WebSnap::imagePathFromUrl(mainFrame()->url().toString());
+    QFile::remove(path);
+    preview.save(path);
 }
 
 
@@ -380,13 +252,13 @@ WebPage *WebPage::createWindow(QWebPage::WebWindowType type)
         kDebug() << "Modal Dialog";
 
     WebTab *w = 0;
-    if (ReKonfig::openTabNoWindow())
+    if (ReKonfig::openLinksInNewWindow())
     {
-        w = rApp->mainWindow()->mainView()->newWebTab(!ReKonfig::openTabsBack());
+        w = rApp->newMainWindow()->mainView()->currentWebTab();
     }
     else
     {
-        w = rApp->newMainWindow()->mainView()->currentWebTab();
+        w = rApp->mainWindow()->mainView()->newWebTab(!ReKonfig::openNewTabsInBackground());
     }
     return w->page();
 }
@@ -410,9 +282,6 @@ void WebPage::handleUnsupportedContent(QNetworkReply *reply)
 
     KIO::Integration::AccessManager::putReplyOnHold(reply);
 
-    // Get suggested file name...
-    extractSuggestedFileName(reply, _suggestedFileName);
-
     // Get mimeType...
     extractMimeType(reply, _mimeType);
 
@@ -420,6 +289,10 @@ void WebPage::handleUnsupportedContent(QNetworkReply *reply)
     if (KParts::BrowserRun::isTextExecutable(_mimeType))
         _mimeType = QL1S("text/plain");
 
+    // Get suggested file name...
+    const KIO::MetaData& data = reply->attribute(static_cast<QNetworkRequest::Attribute>(KIO::AccessManager::MetaData)).toMap();
+    _suggestedFileName = data.value(QL1S("content-disposition-filename"));
+
     kDebug() << "Detected MimeType = " << _mimeType;
     kDebug() << "Suggested File Name = " << _suggestedFileName;
     // ------------------------------------------------
@@ -433,21 +306,34 @@ void WebPage::handleUnsupportedContent(QNetworkReply *reply)
     {
         isLocal
         ? KMessageBox::sorry(view(), i18n("No service can handle this file."))
-        : downloadReply(reply, _suggestedFileName);
+        : downloadUrl(reply->url());
 
         return;
     }
 
-    if (!isLocal)
+    if (isLocal)
+    {
+        // Load outside local files
+        KRun::run(*appService, replyUrl, 0, false, _suggestedFileName);
+    }
+    else
     {
         KParts::BrowserOpenOrSaveQuestion dlg(rApp->mainWindow(), replyUrl, _mimeType);
+
         if (!_suggestedFileName.isEmpty())
             dlg.setSuggestedFileName(_suggestedFileName);
 
+        // read askEmbedOrSave preferences. If we don't have to show dialog and rekonq settings are
+        // to automatically choose download dir, we won't show local dir choose dialog
+        KConfigGroup cg = KConfigGroup(KSharedConfig::openConfig("filetypesrc", KConfig::NoGlobals), QL1S("Notification Messages"));
+        bool hideDialog = cg.readEntry(QL1S("askEmbedOrSave") + _mimeType, false);
+
+        kDebug() << "Hide dialog for " << _mimeType << "? " << hideDialog;
+
         switch (dlg.askEmbedOrSave())
         {
         case KParts::BrowserOpenOrSaveQuestion::Save:
-            downloadReply(reply, _suggestedFileName);
+            rApp->downloadManager()->downloadResource(reply->url(), KIO::MetaData(), view(), !hideDialog, _suggestedFileName);
             return;
 
         case KParts::BrowserOpenOrSaveQuestion::Cancel:
@@ -471,7 +357,7 @@ void WebPage::handleUnsupportedContent(QNetworkReply *reply)
         destUrl.setPath(tempFile.fileName());
         KIO::Job *job = KIO::file_copy(_loadingUrl, destUrl, 0600, KIO::Overwrite);
         job->ui()->setWindow(rApp->mainWindow());
-        connect(job, SIGNAL(result(KJob *)), this, SLOT(copyToTempFileResult(KJob*)));
+        connect(job, SIGNAL(result(KJob*)), this, SLOT(copyToTempFileResult(KJob*)));
         return;
     }
 
@@ -488,7 +374,7 @@ void WebPage::handleUnsupportedContent(QNetworkReply *reply)
         UrlBar *bar = tab->urlBar();
         bar->setQUrl(replyUrl);
 
-        rApp->mainWindow()->updateActions();
+        rApp->mainWindow()->updateHistoryActions();
     }
     else
     {
@@ -521,7 +407,7 @@ void WebPage::loadFinished(bool ok)
     mainFrame()->setZoomFactor(QVariant(value).toReal() / 10);  // Don't allox max +1 values
 
     // Provide site icon. Can this be moved to loadStarted??
-    rApp->iconManager()->provideIcon(this, _loadingUrl);
+    rApp->iconManager()->provideIcon(mainFrame(), _loadingUrl);
 
     // Apply adblock manager hiding rules
     rApp->adblockManager()->applyHidingRules(this);
@@ -541,10 +427,6 @@ void WebPage::manageNetworkErrors(QNetworkReply *reply)
 {
     Q_ASSERT(reply);
 
-    // check suggested file name
-    if (_suggestedFileName.isEmpty())
-        extractSuggestedFileName(reply, _suggestedFileName);
-
     QWebFrame* frame = qobject_cast<QWebFrame *>(reply->request().originatingObject());
     const bool isMainFrameRequest = (frame == mainFrame());
     const bool isLoadingUrlReply = (mainFrame()->url() == reply->url());
@@ -577,14 +459,11 @@ void WebPage::manageNetworkErrors(QNetworkReply *reply)
         // ignore this..
         return;
 
-    case QNetworkReply::ContentAccessDenied:                 // access to remote content denied (similar to HTTP error 401)
+        // WARNING: This is also typical adblocked element error: IGNORE THIS!
+    case QNetworkReply::ContentAccessDenied:                 // access to remote content denied
         break;
 
     case QNetworkReply::UnknownNetworkError:                 // unknown network-related error detected
-        // FIXME: DO WE REALLY NEED THIS???
-        _protHandler.postHandling(reply->request(), frame);
-        return;
-
     case QNetworkReply::ConnectionRefusedError:              // remote server refused connection
     case QNetworkReply::HostNotFoundError:                   // invalid hostname
     case QNetworkReply::TimeoutError:                        // connection time out
@@ -593,7 +472,7 @@ void WebPage::manageNetworkErrors(QNetworkReply *reply)
     case QNetworkReply::ContentNotFoundError:                // remote content not found on server (similar to HTTP error 404)
     case QNetworkReply::ProtocolUnknownError:                // Unknown protocol
     case QNetworkReply::ProtocolInvalidOperationError:       // requested operation is invalid for this protocol
-
+    default:
         kDebug() << "ERROR " << reply->error() << ": " << reply->errorString();
         if (reply->url() == _loadingUrl)
         {
@@ -607,15 +486,11 @@ void WebPage::manageNetworkErrors(QNetworkReply *reply)
                 UrlBar *bar = tab->urlBar();
                 bar->setQUrl(_loadingUrl);
 
-                rApp->mainWindow()->updateActions();
+                rApp->mainWindow()->updateHistoryActions();
             }
         }
         break;
 
-    default:
-        // Nothing to do here..
-        break;
-
     }
 }
 
@@ -676,23 +551,17 @@ QString WebPage::errorPage(QNetworkReply *reply)
 }
 
 
-void WebPage::downloadReply(const QNetworkReply *reply, const QString &suggestedFileName)
-{
-    downloadResource(reply->url(), KIO::MetaData(), view(), suggestedFileName);
-}
-
-
 void WebPage::downloadRequest(const QNetworkRequest &request)
 {
-    downloadResource(request.url(),
-                     request.attribute(static_cast<QNetworkRequest::Attribute>(KIO::AccessManager::MetaData)).toMap(),
-                     view());
+    rApp->downloadManager()->downloadResource(request.url(),
+            request.attribute(static_cast<QNetworkRequest::Attribute>(KIO::AccessManager::MetaData)).toMap(),
+            view());
 }
 
 
 void WebPage::downloadUrl(const KUrl &url)
 {
-    downloadResource(url, KIO::MetaData(), view());
+    rApp->downloadManager()->downloadResource(url, KIO::MetaData(), view());
 }
 
 
@@ -703,28 +572,20 @@ void WebPage::downloadAllContentsWithKGet()
     KUrl relativeUrl;
 
     QWebElementCollection images = mainFrame()->documentElement().findAll("img");
-    foreach(const QWebElement & img, images)
+    Q_FOREACH(const QWebElement & img, images)
     {
         relativeUrl.setEncodedUrl(img.attribute("src").toUtf8(), KUrl::TolerantMode);
         contents << baseUrl.resolved(relativeUrl).toString();
     }
 
     QWebElementCollection links = mainFrame()->documentElement().findAll("a");
-    foreach(const QWebElement & link, links)
+    Q_FOREACH(const QWebElement & link, links)
     {
         relativeUrl.setEncodedUrl(link.attribute("href").toUtf8(), KUrl::TolerantMode);
         contents << baseUrl.resolved(relativeUrl).toString();
     }
 
-    if (!QDBusConnection::sessionBus().interface()->isServiceRegistered("org.kde.kget"))
-    {
-        KToolInvocation::kdeinitExecWait("kget");
-    }
-    QDBusInterface kget("org.kde.kget", "/KGet", "org.kde.kget.main");
-    if (kget.isValid())
-    {
-        kget.call("importLinks", QVariant(contents.toList()));
-    }
+    rApp->downloadManager()->downloadLinksWithKGet(QVariant(contents.toList()));
 }
 
 
@@ -745,16 +606,6 @@ void WebPage::showSSLInfo(QPoint pos)
 }
 
 
-void WebPage::updateImage(bool ok)
-{
-    if (ok)
-    {
-        NewTabPage p(mainFrame());
-        p.snapFinished();
-    }
-}
-
-
 void WebPage::copyToTempFileResult(KJob* job)
 {
     if (job->error())
