@@ -6,7 +6,7 @@
 * Copyright (C) 2008 Dirk Mueller <mueller@kde.org>
 * Copyright (C) 2008 Urs Wolfer <uwolfer @ kde.org>
 * Copyright (C) 2008 Michael Howell <mhowell123@gmail.com>
-* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2012 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2010 by Matthieu Gicquel <matgic78 at gmail dot com>
 *
 *
@@ -56,6 +56,7 @@ public:
     {
         return _networkAnalyzer;
     };
+
     inline void enableNetworkAnalyzer(bool b)
     {
         _networkAnalyzer = b;
@@ -65,6 +66,7 @@ public:
     {
         return _isOnRekonqPage;
     };
+
     inline void setIsOnRekonqPage(bool b)
     {
         _isOnRekonqPage = b;
@@ -101,13 +103,9 @@ private Q_SLOTS:
     void loadStarted();
     void loadFinished(bool);
     void showSSLInfo(QPoint);
-    void updateImage(bool ok);
 
     void copyToTempFileResult(KJob*);
 
-private:
-    void downloadReply(const QNetworkReply *reply, const QString &suggestedFileName = QString());
-
 private:
     QString errorPage(QNetworkReply *reply);
     KUrl _loadingUrl;
