@@ -2,7 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2012 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
 *
 *
@@ -79,6 +79,8 @@ public:
     KUrl url();
     void createPreviewSelectorBar(int index);
 
+    void hideSelectorBar();
+
     bool hasRSSInfo();
 
     bool isPageLoading();
@@ -88,18 +90,21 @@ public:
     {
         return m_part;
     }
+
     void setPart(KParts::ReadOnlyPart *p, const KUrl &u);
 
-    void showMessageBar();
 
 private Q_SLOTS:
     void updateProgress(int progress);
+    void resetProgress();
 
     void createWalletBar(const QString &, const QUrl &);
     void showRSSInfo(const QPoint &pos);
     void showSearchEngine(const QPoint &pos);
     void openSearchEngineAdded();
 
+    void showMessageBar();
+
 Q_SIGNALS:
     void loadProgressing();
     void titleChanged(const QString &);
