@@ -2,7 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2012 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
 *
 *
@@ -34,15 +34,13 @@
 
 // Local Includes
 #include "application.h"
-#include "bookmarkprovider.h"
-#include "bookmarkowner.h"
+#include "bookmarkmanager.h"
 #include "iconmanager.h"
 #include "mainview.h"
 #include "mainwindow.h"
 #include "searchengine.h"
 #include "urlbar.h"
 #include "webpage.h"
-#include "websnap.h"
 #include "webtab.h"
 
 // KDE Includes
@@ -52,6 +50,7 @@
 #include <KMenu>
 #include <KStandardAction>
 #include <KStandardDirs>
+#include <KToolInvocation>
 
 // Qt Includes
 #include <QtCore/QFile>
@@ -66,30 +65,30 @@
 
 WebView::WebView(QWidget* parent)
     : KWebView(parent, false)
-    , m_mousePos(QPoint(0, 0))
     , m_autoScrollTimer(new QTimer(this))
-    , m_vScrollSpeed(0)
-    , m_hScrollSpeed(0)
-    , m_canEnableAutoScroll(true)
-    , m_isAutoScrollEnabled(false)
+    , m_verticalAutoScrollSpeed(0)
+    , m_horizontalAutoScrollSpeed(0)
+    , m_isViewAutoScrolling(false)
     , m_autoScrollIndicator(QPixmap(KStandardDirs::locate("appdata" , "pics/autoscroll.png")))
     , m_smoothScrollTimer(new QTimer(this))
-    , m_smoothScrolling(false)
     , m_dy(0)
     , m_smoothScrollSteps(0)
+    , m_isViewSmoothScrolling(false)
+    , m_accessKeysPressed(false)
+    , m_accessKeysActive(false)
 {
     WebPage *page = new WebPage(this);
     setPage(page);
 
     // download system
-    connect(this, SIGNAL(linkShiftClicked(const KUrl &)), page, SLOT(downloadUrl(const KUrl &)));
+    connect(this, SIGNAL(linkShiftClicked(KUrl)), page, SLOT(downloadUrl(KUrl)));
 
     // middle click || ctrl + click signal
-    connect(this, SIGNAL(linkMiddleOrCtrlClicked(const KUrl &)), this, SLOT(loadUrlInNewTab(const KUrl &)));
+    connect(this, SIGNAL(linkMiddleOrCtrlClicked(KUrl)), this, SLOT(loadUrlInNewTab(KUrl)));
 
     // loadUrl signal
-    connect(this, SIGNAL(loadUrl(const KUrl &, const Rekonq::OpenType &)),
-            rApp, SLOT(loadUrl(const KUrl &, const Rekonq::OpenType &)));
+    connect(this, SIGNAL(loadUrl(KUrl,Rekonq::OpenType)),
+            rApp, SLOT(loadUrl(KUrl,Rekonq::OpenType)));
 
     // Auto scroll timer
     connect(m_autoScrollTimer, SIGNAL(timeout()), this, SLOT(scrollFrameChanged()));
@@ -100,22 +99,23 @@ WebView::WebView(QWidget* parent)
     m_smoothScrollTimer->setInterval(16);
 
     connect(this, SIGNAL(iconChanged()), this, SLOT(changeWindowIcon()));
+    connect(this, SIGNAL(loadStarted()), this, SLOT(loadStarted()));
 }
 
 
 WebView::~WebView()
 {
-    if (m_smoothScrolling)
-        stopScrolling();
-
-    WebPage* p = page();
-
-    QPixmap preview = WebSnap::renderClosingPagePreview(*p);
-    QString path = WebSnap::imagePathFromUrl(p->mainFrame()->url().toString());
-    QFile::remove(path);
-    preview.save(path);
+    if (m_isViewSmoothScrolling)
+        stopSmoothScrolling();
 }
 
+
+void WebView::loadStarted()
+{
+    hideAccessKeys();
+}
+
+
 void WebView::changeWindowIcon()
 {
     if (ReKonfig::useFavicon())
@@ -129,6 +129,7 @@ void WebView::changeWindowIcon()
     }
 }
 
+
 WebPage *WebView::page()
 {
     WebPage *const page = qobject_cast<WebPage *>(KWebView::page());
@@ -137,7 +138,6 @@ WebPage *WebView::page()
 }
 
 
-// TODO: refactor me!
 void WebView::contextMenuEvent(QContextMenuEvent *event)
 {
     QWebHitTestResult result = page()->mainFrame()->hitTestContent(event->pos());
@@ -149,10 +149,97 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
     KAction *inspectAction = new KAction(KIcon("layer-visible-on"), i18n("Inspect Element"), this);
     connect(inspectAction, SIGNAL(triggered(bool)), this, SLOT(inspect()));
 
-    // is a link?
-    if (!result.linkUrl().isEmpty())
+    KAction *sendByMailAction = new KAction(this);
+    sendByMailAction->setIcon(KIcon("mail-send"));
+    connect(sendByMailAction, SIGNAL(triggered(bool)), this, SLOT(sendByMail()));
+
+    // Choose right context
+    int resultHit = 0;
+    if (result.linkUrl().isEmpty())
+        resultHit = WebView::EmptySelection;
+    else
+        resultHit = WebView::LinkSelection;
+
+    if (!result.pixmap().isNull())
+        resultHit |= WebView::ImageSelection;
+
+    if (result.isContentSelected())
+        resultHit = WebView::TextSelection;
+
+    // --------------------------------------------------------------------------------
+    // Ok, let's start filling up the menu...
+
+    // is content editable? Add PASTE
+    if (result.isContentEditable())
     {
-        // link actions
+        menu.addAction(pageAction(KWebPage::Paste));
+        menu.addSeparator();
+    }
+
+    // EMPTY PAGE ACTIONS -------------------------------------------------------------
+    if (resultHit == WebView::EmptySelection)
+    {
+        // send by mail: page url
+        sendByMailAction->setData(page()->currentFrame()->url());
+        sendByMailAction->setText(i18n("Share page url"));
+
+        // navigation
+        QWebHistory *history = page()->history();
+        if (history->canGoBack())
+        {
+            menu.addAction(pageAction(KWebPage::Back));
+        }
+
+        if (history->canGoForward())
+        {
+            menu.addAction(pageAction(KWebPage::Forward));
+        }
+
+        menu.addAction(mainwindow->actionByName("view_redisplay"));
+
+        menu.addSeparator();
+
+        //Frame
+        KActionMenu *frameMenu = new KActionMenu(i18n("Current Frame"), this);
+        frameMenu->addAction(pageAction(KWebPage::OpenFrameInNewWindow));
+
+        a = new KAction(KIcon("document-print-frame"), i18n("Print Frame"), this);
+        connect(a, SIGNAL(triggered()), this, SLOT(printFrame()));
+        frameMenu->addAction(a);
+
+        menu.addAction(frameMenu);
+
+        menu.addSeparator();
+
+        // Page Actions
+        menu.addAction(pageAction(KWebPage::SelectAll));
+
+        menu.addAction(mainwindow->actionByName(KStandardAction::name(KStandardAction::SaveAs)));
+
+        if (!KStandardDirs::findExe("kget").isNull() && ReKonfig::kgetList())
+        {
+            a = new KAction(KIcon("kget"), i18n("List All Links"), this);
+            connect(a, SIGNAL(triggered(bool)), page(), SLOT(downloadAllContentsWithKGet()));
+            menu.addAction(a);
+        }
+
+        menu.addAction(mainwindow->actionByName("page_source"));
+        menu.addAction(inspectAction);
+
+        if (mainwindow->isFullScreen())
+        {
+            menu.addSeparator();
+            menu.addAction(mainwindow->actionByName("fullscreen"));
+        }
+    }
+
+    // LINK ACTIONS -------------------------------------------------------------------
+    if (resultHit & WebView::LinkSelection)
+    {
+        // send by mail: link url
+        sendByMailAction->setData(result.linkUrl());
+        sendByMailAction->setText(i18n("Share link"));
+
         a = new KAction(KIcon("tab-new"), i18n("Open in New &Tab"), this);
         a->setData(result.linkUrl());
         connect(a, SIGNAL(triggered(bool)), this, SLOT(openLinkInNewTab()));
@@ -163,34 +250,57 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
         connect(a, SIGNAL(triggered(bool)), this, SLOT(openLinkInNewWindow()));
         menu.addAction(a);
 
-        a = new KAction(KIcon("bookmark-new"), i18n("&Bookmark this Link"), this);
-        a->setData(result.linkUrl());
-        connect(a, SIGNAL(triggered(bool)), this, SLOT(bookmarkLink()));
-        menu.addAction(a);
-
         menu.addSeparator();
         menu.addAction(pageAction(KWebPage::DownloadLinkToDisk));
         menu.addAction(pageAction(KWebPage::CopyLinkToClipboard));
+    }
+
+    // IMAGE ACTIONS ------------------------------------------------------------------
+    if (resultHit & WebView::ImageSelection)
+    {
+        // send by mail: image url
+        sendByMailAction->setData(result.imageUrl());
+        sendByMailAction->setText(i18n("Share image link"));
+
         menu.addSeparator();
+
+        a = new KAction(KIcon("view-preview"), i18n("&View Image"), this);
+        a->setData(result.imageUrl());
+        connect(a, SIGNAL(triggered(Qt::MouseButtons,Qt::KeyboardModifiers)),
+                this, SLOT(viewImage(Qt::MouseButtons,Qt::KeyboardModifiers)));
+        menu.addAction(a);
+
+        menu.addAction(pageAction(KWebPage::DownloadImageToDisk));
+
+        a = new KAction(KIcon("view-media-visualization"), i18n("&Copy Image Location"), this);
+        a->setData(result.imageUrl());
+        connect(a, SIGNAL(triggered(Qt::MouseButtons,Qt::KeyboardModifiers)), this, SLOT(slotCopyImageLocation()));
+        menu.addAction(a);
     }
 
-    // is content editable && selected? Add CUT
-    if (result.isContentEditable() && result.isContentSelected())
+    // ACTIONS FOR TEXT SELECTION -----------------------------------------------------
+    if (resultHit & WebView::TextSelection)
     {
-        // actions for text selected in field
-        menu.addAction(pageAction(KWebPage::Cut));
-    }
+        // send by mail: text
+        sendByMailAction->setData(selectedText());
+        sendByMailAction->setText(i18n("Share selected text"));
+
+        if (result.isContentEditable())
+        {
+            // actions for text selected in field
+            menu.addAction(pageAction(KWebPage::Cut));
+        }
 
-    // is content selected) Add COPY
-    if (result.isContentSelected())
-    {
         a = pageAction(KWebPage::Copy);
         if (!result.linkUrl().isEmpty())
             a->setText(i18n("Copy Text")); //for link
         else
             a->setText(i18n("Copy"));
         menu.addAction(a);
-        if (selectedText().contains('.') && selectedText().indexOf('.') < selectedText().length() && !selectedText().trimmed().contains(" "))
+
+        if (selectedText().contains('.') && selectedText().indexOf('.') < selectedText().length()
+                && !selectedText().trimmed().contains(" ")
+           )
         {
             QString text = selectedText();
             text = text.trimmed();
@@ -218,17 +328,7 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
                 menu.addSeparator();
             }
         }
-    }
 
-    // is content editable? Add PASTE
-    if (result.isContentEditable())
-    {
-        menu.addAction(pageAction(KWebPage::Paste));
-    }
-
-    // is content selected? Add SEARCH actions
-    if (result.isContentSelected())
-    {
         //Default SearchEngine
         KService::Ptr defaultEngine = SearchEngine::defaultEngine();
         if (defaultEngine) // check if a default engine is set
@@ -241,11 +341,11 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
         }
 
         //All favourite ones
-        KActionMenu *searchMenu = new KActionMenu(KIcon("edit-find"), i18n("Search with"), this);
+        KActionMenu *searchMenu = new KActionMenu(KIcon("edit-find"), i18nc("@title:menu", "Search"), this);
 
-        foreach(const KService::Ptr & engine, SearchEngine::favorites())
+        Q_FOREACH(const KService::Ptr & engine, SearchEngine::favorites())
         {
-            a = new KAction(engine->name(), this);
+            a = new KAction(i18nc("@item:inmenu Search, %1 = search engine", "With %1", engine->name()), this);
             a->setIcon(rApp->iconManager()->iconForUrl(SearchEngine::buildQuery(engine, "")));
             a->setData(engine->entryPath());
             connect(a, SIGNAL(triggered(bool)), this, SLOT(search()));
@@ -260,123 +360,44 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
         {
             menu.addAction(searchMenu);
         }
-
-        menu.addSeparator();
-
-        menu.addAction(inspectAction);
-        // TODO Add translate, show translation
     }
 
-    // is an image?
-    if (!result.pixmap().isNull())
+    // DEFAULT ACTIONs (on the bottom) ------------------------------------------------
+    menu.addSeparator();
+    if (resultHit & WebView::LinkSelection)
     {
-        menu.addSeparator();
-
-        // TODO remove copy_this_image action
-        a = new KAction(KIcon("view-media-visualization"), i18n("&View Image"), this);
-        a->setData(result.imageUrl());
-        connect(a, SIGNAL(triggered(Qt::MouseButtons, Qt::KeyboardModifiers)), this, SLOT(viewImage(Qt::MouseButtons, Qt::KeyboardModifiers)));
+        a = new KAction(KIcon("bookmark-new"), i18n("&Bookmark link"), this);
+        a->setData(result.linkUrl());
+        connect(a, SIGNAL(triggered(bool)), this, SLOT(bookmarkLink()));
         menu.addAction(a);
-
-        menu.addAction(pageAction(KWebPage::DownloadImageToDisk));
-        menu.addAction(pageAction(KWebPage::CopyImageToClipboard));
-
-        a = new KAction(KIcon("view-media-visualization"), i18n("&Copy Image Location"), this);
-        a->setData(result.imageUrl());
-        connect(a, SIGNAL(triggered(Qt::MouseButtons, Qt::KeyboardModifiers)), this, SLOT(slotCopyImageLocation()));
-        menu.addAction(a);
-
-        menu.addSeparator();
-
-        menu.addAction(inspectAction);
     }
-
-    // page actions
-    if (!result.isContentSelected() && result.linkUrl().isEmpty())
+    else
     {
-
-        // navigation
-        QWebHistory *history = page()->history();
-        if (history->canGoBack())
-        {
-            menu.addAction(pageAction(KWebPage::Back));
-        }
-
-        if (history->canGoForward())
-        {
-            menu.addAction(pageAction(KWebPage::Forward));
-        }
-
-        menu.addAction(mainwindow->actionByName("view_redisplay"));
-
-        if (result.pixmap().isNull())
-        {
-            menu.addSeparator();
-
-            if (!ReKonfig::alwaysShowTabBar() && mainwindow->mainView()->count() == 1)
-                menu.addAction(mainwindow->actionByName("new_tab"));
-
-            menu.addAction(mainwindow->actionByName("new_window"));
-
-            menu.addSeparator();
-
-            //Frame
-            KActionMenu *frameMenu = new KActionMenu(i18n("Current Frame"), this);
-
-            frameMenu->addAction(pageAction(KWebPage::OpenFrameInNewWindow));
-
-            a = new KAction(KIcon("document-print-frame"), i18n("Print Frame"), this);
-            connect(a, SIGNAL(triggered()), this, SLOT(printFrame()));
-            frameMenu->addAction(a);
-
-            menu.addAction(frameMenu);
-
-            menu.addSeparator();
-
-            // Page Actions
-            menu.addAction(pageAction(KWebPage::SelectAll));
-
-            menu.addAction(mainwindow->actionByName(KStandardAction::name(KStandardAction::SaveAs)));
-
-            if (!KStandardDirs::findExe("kget").isNull() && ReKonfig::kgetList())
-            {
-                a = new KAction(KIcon("kget"), i18n("List All Links"), this);
-                connect(a, SIGNAL(triggered(bool)), page(), SLOT(downloadAllContentsWithKGet()));
-                menu.addAction(a);
-            }
-
-            menu.addAction(mainwindow->actionByName("page_source"));
-            menu.addAction(inspectAction);
-
-            a = rApp->bookmarkProvider()->actionByName("rekonq_add_bookmark");
-            menu.addAction(a);
-        }
-
-        if (mainwindow->isFullScreen())
-        {
-            menu.addSeparator();
-            menu.addAction(mainwindow->actionByName("fullscreen"));
-        }
+        a = rApp->bookmarkManager()->actionByName("rekonq_add_bookmark");
+        menu.addAction(a);
     }
+    menu.addAction(sendByMailAction);
+    menu.addAction(inspectAction);
 
+    // finally launch the menu...
     menu.exec(mapToGlobal(event->pos()));
 }
 
 
 void WebView::mousePressEvent(QMouseEvent *event)
 {
-    if (m_isAutoScrollEnabled)
+    if (m_isViewAutoScrolling)
     {
-        m_vScrollSpeed = 0;
-        m_hScrollSpeed = 0;
+        m_verticalAutoScrollSpeed = 0;
+        m_horizontalAutoScrollSpeed = 0;
         m_autoScrollTimer->stop();
-        m_isAutoScrollEnabled = false;
+        m_isViewAutoScrolling = false;
         update();
         return;
     }
 
     QWebHitTestResult result = page()->mainFrame()->hitTestContent(event->pos());
-    m_canEnableAutoScroll = ReKonfig::middleClickAction() == 0 && !result.isContentEditable()  && result.linkUrl().isEmpty();
+    bool weCanDoMiddleClickActions = !result.isContentEditable()  && result.linkUrl().isEmpty();
 
     switch (event->button())
     {
@@ -389,35 +410,45 @@ void WebView::mousePressEvent(QMouseEvent *event)
         break;
 
     case Qt::MidButton:
-        if (m_canEnableAutoScroll
-                && !m_isAutoScrollEnabled
-                && !page()->currentFrame()->scrollBarGeometry(Qt::Horizontal).contains(event->pos())
-                && !page()->currentFrame()->scrollBarGeometry(Qt::Vertical).contains(event->pos()))
+        switch (ReKonfig::middleClickAction())
         {
-            if (!page()->currentFrame()->scrollBarGeometry(Qt::Horizontal).isNull()
-                    || !page()->currentFrame()->scrollBarGeometry(Qt::Vertical).isNull())
+        case 0: // AutoScroll
+            if (weCanDoMiddleClickActions
+                    && !m_isViewAutoScrolling
+                    && !page()->currentFrame()->scrollBarGeometry(Qt::Horizontal).contains(event->pos())
+                    && !page()->currentFrame()->scrollBarGeometry(Qt::Vertical).contains(event->pos()))
             {
-                m_clickPos = event->pos();
-                m_isAutoScrollEnabled = true;
-                update();
+                if (!page()->currentFrame()->scrollBarGeometry(Qt::Horizontal).isNull()
+                        || !page()->currentFrame()->scrollBarGeometry(Qt::Vertical).isNull())
+                {
+                    m_clickPos = event->pos();
+                    m_isViewAutoScrolling = true;
+                    update();
+                }
             }
-        }
+            break;
 
-        if (ReKonfig::middleClickAction() == 1 && result.linkUrl().isEmpty() && !result.isContentEditable())
-        {
-            const QString clipboardContent = rApp->clipboard()->text();
-
-            if (clipboardContent.isEmpty())
-                break;
-
-            if (QUrl::fromUserInput(clipboardContent).isValid())
-                loadUrl(clipboardContent, Rekonq::CurrentTab);
-            else // Search with default Engine
+        case 1: // Load Clipboard URL
+            if (weCanDoMiddleClickActions)
             {
-                KService::Ptr defaultEngine = SearchEngine::defaultEngine();
-                if (defaultEngine) // check if a default engine is set
-                    loadUrl(KUrl(SearchEngine::buildQuery(defaultEngine, clipboardContent)), Rekonq::CurrentTab);
+                const QString clipboardContent = rApp->clipboard()->text();
+
+                if (clipboardContent.isEmpty())
+                    break;
+
+                if (QUrl::fromUserInput(clipboardContent).isValid())
+                    loadUrl(clipboardContent, Rekonq::CurrentTab);
+                else // Search with default Engine
+                {
+                    KService::Ptr defaultEngine = SearchEngine::defaultEngine();
+                    if (defaultEngine) // check if a default engine is set
+                        loadUrl(KUrl(SearchEngine::buildQuery(defaultEngine, clipboardContent)), Rekonq::CurrentTab);
+                }
             }
+            break;
+
+        default: // Do Nothing
+            break;
         }
         break;
 
@@ -430,13 +461,13 @@ void WebView::mousePressEvent(QMouseEvent *event)
 
 void WebView::mouseMoveEvent(QMouseEvent *event)
 {
-    m_mousePos = event->pos();
+    QPoint mousePos = event->pos();
 
-    if (m_isAutoScrollEnabled)
+    if (m_isViewAutoScrolling)
     {
-        QPoint r = m_mousePos - m_clickPos;
-        m_hScrollSpeed = r.x() / 2;  // you are too fast..
-        m_vScrollSpeed = r.y() / 2;
+        QPoint r = mousePos - m_clickPos;
+        m_horizontalAutoScrollSpeed = r.x() / 2;  // you are too fast..
+        m_verticalAutoScrollSpeed = r.y() / 2;
         if (!m_autoScrollTimer->isActive())
             m_autoScrollTimer->start();
 
@@ -466,10 +497,10 @@ void WebView::dropEvent(QDropEvent *event)
     if (event->mimeData()->hasFormat("application/rekonq-bookmark"))
     {
         QByteArray addresses = event->mimeData()->data("application/rekonq-bookmark");
-        KBookmark bookmark =  rApp->bookmarkProvider()->bookmarkManager()->findByAddress(QString::fromLatin1(addresses.data()));
+        KBookmark bookmark =  rApp->bookmarkManager()->findByAddress(QString::fromLatin1(addresses.data()));
         if (bookmark.isGroup())
         {
-            rApp->bookmarkProvider()->bookmarkOwner()->openFolderinTabs(bookmark.toGroup());
+            rApp->bookmarkManager()->openFolderinTabs(bookmark.toGroup());
         }
         else
         {
@@ -501,7 +532,7 @@ void WebView::paintEvent(QPaintEvent* event)
 {
     KWebView::paintEvent(event);
 
-    if (m_isAutoScrollEnabled)
+    if (m_isViewAutoScrolling)
     {
         QPoint centeredPoint = m_clickPos;
         centeredPoint.setX(centeredPoint.x() - m_autoScrollIndicator.width() / 2);
@@ -545,6 +576,7 @@ void WebView::viewImage(Qt::MouseButtons buttons, Qt::KeyboardModifiers modifier
     }
 }
 
+
 void WebView::slotCopyImageLocation()
 {
     KAction *a = qobject_cast<KAction*>(sender());
@@ -586,113 +618,207 @@ void WebView::bookmarkLink()
     KAction *a = qobject_cast<KAction*>(sender());
     KUrl url(a->data().toUrl());
 
-    rApp->bookmarkProvider()->rootGroup().addBookmark(url.prettyUrl(), url);
-    rApp->bookmarkProvider()->bookmarkManager()->emitChanged();
+    rApp->bookmarkManager()->rootGroup().addBookmark(url.prettyUrl(), url);
+    rApp->bookmarkManager()->emitChanged();
 }
 
 
 void WebView::keyPressEvent(QKeyEvent *event)
 {
+    // If CTRL was hit, be prepared for access keys
+    if (ReKonfig::accessKeysEnabled()
+            && !m_accessKeysActive
+            && event->key() == Qt::Key_Control
+            && !(event->modifiers() & ~Qt::ControlModifier)
+       )
+    {
+        m_accessKeysPressed = true;
+        event->accept();
+        return;
+    }
+
+    const QString tagName = page()->mainFrame()->evaluateJavaScript("document.activeElement.tagName").toString();
+
     if (event->modifiers() == Qt::ControlModifier)
     {
         if (event->key() == Qt::Key_C)
         {
             triggerPageAction(KWebPage::Copy);
+            event->accept();
             return;
         }
 
         if (event->key() == Qt::Key_A)
         {
             triggerPageAction(KWebPage::SelectAll);
+            event->accept();
             return;
         }
-    }
 
-    if (!m_canEnableAutoScroll)
-    {
-        KWebView::keyPressEvent(event);
-        return;
+        // CTRL + RETURN: open link into another tab
+        if (event->key() == Qt::Key_Return && tagName == QL1S("A"))
+        {
+            KUrl u = KUrl(page()->mainFrame()->evaluateJavaScript("document.activeElement.attributes[\"href\"].value").toString());
+            emit loadUrl(u, Rekonq::NewTab);
+            event->accept();
+            return;
+        }
     }
 
     // Auto Scrolling
-    if (event->modifiers() == Qt::ShiftModifier)
+    if (event->modifiers() == Qt::ShiftModifier
+            && tagName != QL1S("INPUT")
+            && tagName != QL1S("TEXTAREA")
+       )
     {
-        if (event->key() == Qt::Key_Up)
+        // NOTE and FIXME
+        // This check is doabled because it presents strange behavior: QtWebKit check works well in pages like gmail
+        // and fails on sites like g+. The opposite is true for javascript check.
+        // Please, help me finding the right way to check this EVERY TIME.
+        bool isContentEditableQW = page()->mainFrame()->hitTestContent(QCursor::pos()).isContentEditable();
+        bool isContentEditableJS = page()->mainFrame()->evaluateJavaScript("document.activeElement.isContentEditable").toBool();
+
+        if (!isContentEditableQW && !isContentEditableJS)
         {
-            m_vScrollSpeed--;
-            if (!m_autoScrollTimer->isActive())
-                m_autoScrollTimer->start();
-            return;
+            if (event->key() == Qt::Key_Up)
+            {
+                m_verticalAutoScrollSpeed--;
+                if (!m_autoScrollTimer->isActive())
+                    m_autoScrollTimer->start();
+
+                event->accept();
+                return;
+            }
+
+            if (event->key() == Qt::Key_Down)
+            {
+                m_verticalAutoScrollSpeed++;
+                if (!m_autoScrollTimer->isActive())
+                    m_autoScrollTimer->start();
+
+                event->accept();
+                return;
+            }
+
+            if (event->key() == Qt::Key_Right)
+            {
+                m_horizontalAutoScrollSpeed++;
+                if (!m_autoScrollTimer->isActive())
+                    m_autoScrollTimer->start();
+
+                event->accept();
+                return;
+            }
+
+            if (event->key() == Qt::Key_Left)
+            {
+                m_horizontalAutoScrollSpeed--;
+                if (!m_autoScrollTimer->isActive())
+                    m_autoScrollTimer->start();
+
+                event->accept();
+                return;
+            }
+
+            if (m_autoScrollTimer->isActive())
+            {
+                m_autoScrollTimer->stop();
+                event->accept();
+                return;
+            }
+            else
+            {
+                if (m_verticalAutoScrollSpeed || m_horizontalAutoScrollSpeed)
+                {
+                    m_autoScrollTimer->start();
+                    event->accept();
+                    return;
+                }
+            }
         }
 
-        if (event->key() == Qt::Key_Down)
-        {
-            m_vScrollSpeed++;
-            if (!m_autoScrollTimer->isActive())
-                m_autoScrollTimer->start();
-            return;
-        }
+        // if you arrived here, I hope it means SHIFT has been pressed NOT for autoscroll management...
+    }
 
-        if (event->key() == Qt::Key_Right)
-        {
-            m_hScrollSpeed++;
-            if (!m_autoScrollTimer->isActive())
-                m_autoScrollTimer->start();
-            return;
-        }
-
-        if (event->key() == Qt::Key_Left)
-        {
-            m_hScrollSpeed--;
-            if (!m_autoScrollTimer->isActive())
-                m_autoScrollTimer->start();
-            return;
-        }
-
-        if (m_autoScrollTimer->isActive())
-        {
-            m_autoScrollTimer->stop();
-        }
-        else
-        {
-            if (m_vScrollSpeed || m_hScrollSpeed)
-                m_autoScrollTimer->start();
-        }
+    if (ReKonfig::accessKeysEnabled() && m_accessKeysActive)
+    {
+        hideAccessKeys();
+        event->accept();
+        return;
     }
 
     // vi-like navigation
     if (ReKonfig::enableViShortcuts())
     {
-        const QString tagName = page()->mainFrame()->evaluateJavaScript("document.activeElement.tagName").toString();
-        if (tagName != QL1S("INPUT") && tagName != QL1S("TEXTAREA") && event->modifiers() == Qt::NoModifier)
+        if (event->modifiers() == Qt::NoModifier
+                && tagName != QL1S("INPUT")
+                && tagName != QL1S("TEXTAREA")
+           )
         {
-            switch (event->key())
+            // See note up!
+            bool isContentEditableQW = page()->mainFrame()->hitTestContent(QCursor::pos()).isContentEditable();
+            bool isContentEditableJS = page()->mainFrame()->evaluateJavaScript("document.activeElement.isContentEditable").toBool();
+
+            if (!isContentEditableQW && !isContentEditableJS)
             {
-            case Qt::Key_J:
-                event->accept();
-                event = new QKeyEvent(QEvent::KeyPress, Qt::Key_Down, Qt::NoModifier);
-                break;
-            case Qt::Key_K:
-                event->accept();
-                event = new QKeyEvent(QEvent::KeyPress, Qt::Key_Up, Qt::NoModifier);
-                break;
-            case Qt::Key_L:
-                event->accept();
-                event = new QKeyEvent(QEvent::KeyPress, Qt::Key_Right, Qt::NoModifier);
-                break;
-            case Qt::Key_H:
-                event->accept();
-                event = new QKeyEvent(QEvent::KeyPress, Qt::Key_Left, Qt::NoModifier);
-                break;
-            default:
-                break;
+                switch (event->key())
+                {
+                case Qt::Key_J:
+                    event->accept();
+                    event = new QKeyEvent(QEvent::KeyPress, Qt::Key_Down, Qt::NoModifier);
+                    break;
+                case Qt::Key_K:
+                    event->accept();
+                    event = new QKeyEvent(QEvent::KeyPress, Qt::Key_Up, Qt::NoModifier);
+                    break;
+                case Qt::Key_L:
+                    event->accept();
+                    event = new QKeyEvent(QEvent::KeyPress, Qt::Key_Right, Qt::NoModifier);
+                    break;
+                case Qt::Key_H:
+                    event->accept();
+                    event = new QKeyEvent(QEvent::KeyPress, Qt::Key_Left, Qt::NoModifier);
+                    break;
+                default:
+                    break;
+                }
             }
         }
     }
+
     KWebView::keyPressEvent(event);
 }
 
 
+void WebView::keyReleaseEvent(QKeyEvent *event)
+{
+    // access keys management
+    if (ReKonfig::accessKeysEnabled())
+    {
+        if (m_accessKeysPressed && event->key() != Qt::Key_Control)
+            m_accessKeysPressed = false;
+
+        if (m_accessKeysPressed && !(event->modifiers() & Qt::ControlModifier))
+        {
+            kDebug() << "Shotting access keys";
+            QTimer::singleShot(200, this, SLOT(accessKeyShortcut()));
+            event->accept();
+            return;
+        }
+        else
+        {
+            checkForAccessKey(event);
+            kDebug() << "Hiding access keys";
+            hideAccessKeys();
+            event->accept();
+            return;
+        }
+    }
+
+    KWebView::keyReleaseEvent(event);
+}
+
+
 void WebView::wheelEvent(QWheelEvent *event)
 {
     if (event->orientation() == Qt::Vertical || !ReKonfig::hScrollWheelHistory())
@@ -720,16 +846,15 @@ void WebView::wheelEvent(QWheelEvent *event)
         }
         else if (ReKonfig::smoothScrolling() && prevPos != newPos)
         {
-
             page()->currentFrame()->setScrollPosition(QPoint(page()->currentFrame()->scrollPosition().x(), prevPos));
 
-            if ((event->delta() > 0) != !m_scrollBottom)
-                stopScrolling();
+            if ((event->delta() > 0) != !m_smoothScrollBottomReached)
+                stopSmoothScrolling();
 
             if (event->delta() > 0)
-                m_scrollBottom = false;
+                m_smoothScrollBottomReached = false;
             else
-                m_scrollBottom = true;
+                m_smoothScrollBottomReached = true;
 
 
             setupSmoothScrolling(abs(newPos - prevPos));
@@ -770,16 +895,16 @@ void WebView::loadUrlInNewTab(const KUrl &url)
 void WebView::scrollFrameChanged()
 {
     // do the scrolling
-    page()->currentFrame()->scroll(m_hScrollSpeed, m_vScrollSpeed);
+    page()->currentFrame()->scroll(m_horizontalAutoScrollSpeed, m_verticalAutoScrollSpeed);
 
     // check if we reached the end
     int y = page()->currentFrame()->scrollPosition().y();
     if (y == 0 || y == page()->currentFrame()->scrollBarMaximum(Qt::Vertical))
-        m_vScrollSpeed = 0;
+        m_verticalAutoScrollSpeed = 0;
 
     int x = page()->currentFrame()->scrollPosition().x();
     if (x == 0 || x == page()->currentFrame()->scrollBarMaximum(Qt::Horizontal))
-        m_hScrollSpeed = 0;
+        m_horizontalAutoScrollSpeed = 0;
 }
 
 
@@ -791,7 +916,7 @@ void WebView::setupSmoothScrolling(int posY)
 
     if (m_dy <= 0)
     {
-        stopScrolling();
+        stopSmoothScrolling();
         return;
     }
 
@@ -806,9 +931,9 @@ void WebView::setupSmoothScrolling(int posY)
 
     m_smoothScrollTime.start();
 
-    if (!m_smoothScrolling)
+    if (!m_isViewSmoothScrolling)
     {
-        m_smoothScrolling = true;
+        m_isViewSmoothScrolling = true;
         m_smoothScrollTimer->start();
         scrollTick();
     }
@@ -819,7 +944,7 @@ void WebView::scrollTick()
 {
     if (m_dy == 0)
     {
-        stopScrolling();
+        stopSmoothScrolling();
         return;
     }
 
@@ -849,18 +974,18 @@ void WebView::scrollTick()
         m_smoothScrollSteps--;
     }
 
-    if (m_scrollBottom)
+    if (m_smoothScrollBottomReached)
         page()->currentFrame()->scroll(0, scroll_y);
     else
         page()->currentFrame()->scroll(0, -scroll_y);
 }
 
 
-void WebView::stopScrolling()
+void WebView::stopSmoothScrolling()
 {
     m_smoothScrollTimer->stop();
     m_dy = 0;
-    m_smoothScrolling = false;
+    m_isViewSmoothScrolling = false;
 }
 
 
@@ -880,3 +1005,190 @@ void WebView::dragMoveEvent(QDragMoveEvent *event)
     else
         KWebView::dragMoveEvent(event);
 }
+
+
+void WebView::hideAccessKeys()
+{
+    if (!m_accessKeyLabels.isEmpty())
+    {
+        for (int i = 0; i < m_accessKeyLabels.count(); ++i)
+        {
+            QLabel *label = m_accessKeyLabels[i];
+            label->hide();
+            label->deleteLater();
+        }
+        m_accessKeyLabels.clear();
+        m_accessKeyNodes.clear();
+        update();
+    }
+}
+
+
+void WebView::showAccessKeys()
+{
+    QStringList supportedElement;
+    supportedElement << QLatin1String("a")
+                     << QLatin1String("input")
+                     << QLatin1String("area")
+                     << QLatin1String("button")
+                     << QLatin1String("label")
+                     << QLatin1String("legend")
+                     << QLatin1String("textarea");
+
+    QList<QChar> unusedKeys;
+    for (char c = 'A'; c <= 'Z'; ++c)
+        unusedKeys << QLatin1Char(c);
+    for (char c = '0'; c <= '9'; ++c)
+        unusedKeys << QLatin1Char(c);
+
+    QRect viewport = QRect(page()->mainFrame()->scrollPosition(), page()->viewportSize());
+    // Priority first goes to elements with accesskey attributes
+    QList<QWebElement> alreadyLabeled;
+    Q_FOREACH(const QString & elementType, supportedElement)
+    {
+        QList<QWebElement> result = page()->mainFrame()->findAllElements(elementType).toList();
+        Q_FOREACH(const QWebElement & element, result)
+        {
+            const QRect geometry = element.geometry();
+            if (geometry.size().isEmpty()
+                    || !viewport.contains(geometry.topLeft()))
+            {
+                continue;
+            }
+            QString accessKeyAttribute = element.attribute(QLatin1String("accesskey")).toUpper();
+            if (accessKeyAttribute.isEmpty())
+                continue;
+            QChar accessKey;
+            for (int i = 0; i < accessKeyAttribute.count(); i += 2)
+            {
+                const QChar &possibleAccessKey = accessKeyAttribute[i];
+                if (unusedKeys.contains(possibleAccessKey))
+                {
+                    accessKey = possibleAccessKey;
+                    break;
+                }
+            }
+            if (accessKey.isNull())
+            {
+                continue;
+            }
+            unusedKeys.removeOne(accessKey);
+            makeAccessKeyLabel(accessKey, element);
+            alreadyLabeled.append(element);
+        }
+    }
+
+    // Pick an access key first from the letters in the text and then from the
+    // list of unused access keys
+    Q_FOREACH(const QString & elementType, supportedElement)
+    {
+        QWebElementCollection result = page()->mainFrame()->findAllElements(elementType);
+        Q_FOREACH(const QWebElement & element, result)
+        {
+            const QRect geometry = element.geometry();
+            if (unusedKeys.isEmpty()
+                    || alreadyLabeled.contains(element)
+                    || geometry.size().isEmpty()
+                    || !viewport.contains(geometry.topLeft()))
+            {
+                continue;
+            }
+            QChar accessKey;
+            QString text = element.toPlainText().toUpper();
+            for (int i = 0; i < text.count(); ++i)
+            {
+                const QChar &c = text.at(i);
+                if (unusedKeys.contains(c))
+                {
+                    accessKey = c;
+                    break;
+                }
+            }
+            if (accessKey.isNull())
+                accessKey = unusedKeys.takeFirst();
+            unusedKeys.removeOne(accessKey);
+            makeAccessKeyLabel(accessKey, element);
+        }
+    }
+}
+
+
+void WebView::makeAccessKeyLabel(const QChar &accessKey, const QWebElement &element)
+{
+    QLabel *label = new QLabel(this);
+    label->setText(QString(QLatin1String("<qt><b>%1</b>")).arg(accessKey));
+
+    label->setAutoFillBackground(true);
+    label->setFrameStyle(QFrame::Box | QFrame::Plain);
+    QPoint point = element.geometry().center();
+    point -= page()->mainFrame()->scrollPosition();
+    label->move(point);
+    label->show();
+    point.setX(point.x() - label->width() / 2);
+    label->move(point);
+    m_accessKeyLabels.append(label);
+    m_accessKeyNodes[accessKey] = element;
+}
+
+
+bool WebView::checkForAccessKey(QKeyEvent *event)
+{
+    if (m_accessKeyLabels.isEmpty())
+        return false;
+
+    QString text = event->text();
+    if (text.isEmpty())
+        return false;
+    QChar key = text.at(0).toUpper();
+    bool handled = false;
+    if (m_accessKeyNodes.contains(key))
+    {
+        QWebElement element = m_accessKeyNodes[key];
+        QPoint p = element.geometry().center();
+        QWebFrame *frame = element.webFrame();
+        Q_ASSERT(frame);
+        do
+        {
+            p -= frame->scrollPosition();
+            frame = frame->parentFrame();
+        }
+        while (frame && frame != page()->mainFrame());
+        QMouseEvent pevent(QEvent::MouseButtonPress, p, Qt::LeftButton, 0, 0);
+        rApp->sendEvent(this, &pevent);
+        QMouseEvent revent(QEvent::MouseButtonRelease, p, Qt::LeftButton, 0, 0);
+        rApp->sendEvent(this, &revent);
+        handled = true;
+    }
+
+    kDebug() << "checking for access keys: " << handled;
+    return handled;
+}
+
+
+void WebView::accessKeyShortcut()
+{
+    if (!hasFocus()
+            || !m_accessKeysPressed
+            || !ReKonfig::accessKeysEnabled())
+        return;
+    if (m_accessKeyLabels.isEmpty())
+    {
+        showAccessKeys();
+    }
+    else
+    {
+        hideAccessKeys();
+    }
+    m_accessKeysPressed = false;
+}
+
+
+void WebView::sendByMail()
+{
+    KAction *a = qobject_cast<KAction*>(sender());
+    QString url = a->data().toString();
+    kDebug() << "URL " << url;
+
+    KToolInvocation::invokeMailer("", "", "", "", url);
+}
+
